package com.jpsoft.campus.modules.base.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.jpsoft.campus.modules.base.entity.ApplicationMiddle;
import com.jpsoft.campus.modules.base.entity.ApportionDetail;
import com.jpsoft.campus.modules.base.entity.SchoolNumber;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotBlank;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

@Data
public class ApplicationMiddleDTO implements Serializable {
    @ApiModelProperty(value = "申请表单编号")
    private String applicationId;

    @NotBlank(message = "报名类型不能为空")
    @ApiModelProperty(value = "报名类型(A,B,C,D,E)")
    private String type;

    @ApiModelProperty(value = "学生编号")
    private String studentId;

    @NotBlank(message = "姓名不能为空")
    @ApiModelProperty(value = "姓名")
    private String name;

    @ApiModelProperty(value = "曾用名")
    private String oldName;

    @ApiModelProperty(value = "性别:1男2女")
    private Integer sex;

    @NotBlank(message = "证件类型不能为空")
    @ApiModelProperty(value = "证件类型")
    private String cardType;

    @NotBlank(message = "身份证不能为空")
    @ApiModelProperty(value = "身份证")
    private String idCard;

    @ApiModelProperty(value = "地址")
    private String address;

    @ApiModelProperty(value = "民族")
    private String nation;

    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @ApiModelProperty(value = "出生年月")
    private Date birthday;

    @ApiModelProperty(value = "证件上传地址")
    private String documentUrl;

    @ApiModelProperty(value = "所属街道")
    private String streetId;

    @ApiModelProperty(value = "所在社区")
    private String communityId;

    @NotBlank(message = "选择学校不能为空")
    @ApiModelProperty(value = "选择学校")
    private String schoolId;

    @ApiModelProperty(value = "择校原因")
    private String chooseReason;

    @ApiModelProperty(value = "关系")
    private String relationship;

    @ApiModelProperty(value = "报名资料数据目录")
    private String categoryId;

    @ApiModelProperty(value = "报名相关资料")
    private List<AttachmentDTO> registrationFileList;

    @ApiModelProperty(value = "学生证明附件")
    private List<AttachmentDTO> studentCertificationFileList;

    @ApiModelProperty(value = "关系名称")
    private String relationshipName;

    @ApiModelProperty(value = "报名类型名称")
    private String typeName;

    @ApiModelProperty(value = "审批状态名称")
    private String statusName;

    @ApiModelProperty(value = "审批状态")
    private String status;

    @ApiModelProperty(value = "学校名称")
    private String schoolName;

    @ApiModelProperty(value = "学生类型（1：小学，2：中学，3：高中）")
    private String studentType;

    /**
     * 创建人
     */
    @ApiModelProperty(value = "创建人")
    private String createBy;
    /**
     * 创建时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm", timezone = "GMT+8")
    @ApiModelProperty(value = "创建时间")
    private Date createTime;
    /**
     * 更新人
     */
    @ApiModelProperty(value = "更新人")
    private String updateBy;
    /**
     * 更新时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm", timezone = "GMT+8")
    @ApiModelProperty(value = "更新时间")
    private Date updateTime;
    /**
     * 删除标示
     */
    @ApiModelProperty(value = "删除标示")
    private Boolean delFlag;

    @ApiModelProperty(value = "登记人ID")
    private String personId;

    @ApiModelProperty(value = "发送微信消息")
    private Boolean isSend;

    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm", timezone = "GMT+8")
    @ApiModelProperty(value = "微信消息发送时间")
    private Date sendTime;

    @ApiModelProperty(value = "报名资料数据目录")
    private String categoryName;

    @ApiModelProperty(value = "录取编号")
    private String offerId;

    @ApiModelProperty(value = "录取学校名称")
    private String offerName;

    @ApiModelProperty(value = "派位信息")
    private ApportionDetail apportionDetail;

    /**
     *录取编号
     */
    @ApiModelProperty(value = "录取编号")
    private String offerCode;

    @ApiModelProperty(value = "入学通知书")
    private String offerImg;

    @ApiModelProperty(value = "登记人名称")
    private String personName;
    @ApiModelProperty(value = "登记人电话")
    private String personPhone;

    private String studentNumber;

    private String oldClass;

    @ApiModelProperty(value = "转校插班申报的年级ID")
    private String schoolNumberId;

    @ApiModelProperty(value = "转校插班申报的年级")
    private SchoolNumber schoolNumber;
}
