package com.jpsoft.campus.modules.common.utils;

import net.sf.json.JSON;
import net.sf.json.JSONObject;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map;

public class BaiduOCR {
    /**
     * 获取API访问token
     * 该token有一定的有效期，需要自行管理，当失效时需重新获取.
     * @param ak - 百度云官网获取的 API Key
     * @param sk - 百度云官网获取的 Securet Key
     * @return assess_token 示例：
     * "24.460da4889caad24cccdb1fea17221975.2592000.1491995545.282335-1234567"
     */
    public static JSONObject getAuth(String ak, String sk) {
        // 获取token地址
        String authHost = "https://aip.baidubce.com/oauth/2.0/token?";
        String getAccessTokenUrl = authHost
                // 1. grant_type为固定参数
                + "grant_type=client_credentials"
                // 2. 官网获取的 API Key
                + "&client_id=" + ak
                // 3. 官网获取的 Secret Key
                + "&client_secret=" + sk;

        try {
            URL realUrl = new URL(getAccessTokenUrl);
            // 打开和URL之间的连接
            HttpURLConnection connection = (HttpURLConnection) realUrl.openConnection();
            connection.setRequestMethod("GET");
            connection.connect();

            // 定义 BufferedReader输入流来读取URL的响应
            BufferedReader in = new BufferedReader(new InputStreamReader(connection.getInputStream()));
            String result = "";
            String line;

            while ((line = in.readLine()) != null) {
                result += line;
            }

            System.err.println("result:" + result);
            JSONObject jsonObject = JSONObject.fromObject(result);

            return jsonObject;
        } catch (Exception e) {
            System.err.printf("获取token失败！");
            e.printStackTrace(System.err);
        }

        return null;
    }

    public static JSONObject idcard(String accessToken,String base64Data) {
        // 请求url
        String url = "https://aip.baidubce.com/rest/2.0/ocr/v1/idcard";

        try {
//            byte[] imgData = FileUtil.readFileByBytes(filePath);
//            String imgStr = Base64.encode(imgData);
            String imgParam = URLEncoder.encode(base64Data, "UTF-8");

            String param = "id_card_side=" + "front" + "&image=" + imgParam;

            String result = HttpUtil.post(url, accessToken, param);
            JSONObject jsonObject = JSONObject.fromObject(result);

            System.out.println(result);

            return jsonObject;
        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }

    public static JSONObject householdRegister(String accessToken, String base64Data) {
        // 请求url
        String url = "https://aip.baidubce.com/rest/2.0/ocr/v1/household_register";

        try {
            // 本地文件路径
//            byte[] imgData = FileUtil.readFileByBytes(filePath);
//            String imgStr = Base64.encode(imgData);
            String imgParam = URLEncoder.encode(base64Data, "UTF-8");

            String param = "image=" + imgParam;

            String result = HttpUtil.post(url, accessToken, param);
            JSONObject jsonObject = JSONObject.fromObject(result);

            System.out.println(result);

            return jsonObject;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }
}
