package com.jpsoft.campus.modules.common.utils;

import com.jpsoft.campus.modules.wechat.entity.AccessToken;
import lombok.extern.slf4j.Slf4j;
import net.sf.json.JSONObject;
import org.apache.commons.lang3.StringUtils;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
public class WechatMessageUtil {
    public static final String send_template = "https://api.weixin.qq.com/cgi-bin/message/template/send?access_token=ACCESS_TOKEN";

    /**
     * 微信发送模版
     *
     * @param sendData   发送数据
     * @param templateId 模版Id
     * @param openId     发送人
     * @param url        详情跳转地址
     * @return
     */
    public static boolean sendTemplate(JSONObject sendData,String tokenUrl, String appId, String appSecret, String templateId, String openId, String url) {
        boolean result = false;

        AccessToken accessToken = WeixinUtil.getCommonAccessToken(tokenUrl, appId, appSecret);

        if (accessToken != null) {
            //发送模版内容
            String sendTemplateUrl = send_template.replace("ACCESS_TOKEN", accessToken.getToken());

            JSONObject sendTemplateData = new JSONObject();

            sendTemplateData.put("touser", openId);
            sendTemplateData.put("template_id", templateId);
            sendTemplateData.put("url", url);
            sendTemplateData.put("data", sendData);

            log.warn("模版发送Id>>>>" + templateId + ">>>模版发送数据>>>>>" + sendTemplateData.toString());

            String sendTemplateRet = HttpConnectionUtil.requestByPost(sendTemplateUrl, sendTemplateData.toString());

            log.warn("模版发送返回数据>>>>>" + sendTemplateRet);

            if (StringUtils.isNotBlank(sendTemplateRet)) {
                JSONObject sendTemplateJson = JSONObject.fromObject(sendTemplateRet);

                if ("0".equals(sendTemplateJson.getString("errcode"))) {
                    //发送成功
                    result = true;
                }
            }
        }

        return result;
    }

    public static String sendNativeTemplate(String tokenUrl,String appId,String appSecret,String templateId, String openId,JSONObject sendData,String urlPath) {
        String result = "";

        try {
            AccessToken accessToken = WeixinUtil.getCommonAccessToken(tokenUrl,appId, appSecret);

            if (accessToken == null) {
                throw new Exception("token无法获取");
            }

            //发送模版内容
            String sendTemplateUrl = send_template.replace("ACCESS_TOKEN", accessToken.getToken());

            JSONObject sendTemplateData = new JSONObject();

            sendTemplateData.put("touser", openId);
            sendTemplateData.put("template_id", templateId);
            sendTemplateData.put("url", urlPath);
            sendTemplateData.put("data", sendData);

            URL url = new URL(sendTemplateUrl);
            HttpURLConnection conn = (HttpURLConnection) url.openConnection();
            conn.setRequestMethod("POST");
            conn.setDoOutput(true);
            conn.setDoInput(true);
            conn.setUseCaches(false);
            conn.setRequestProperty("Connection", "Keep-Alive");
            conn.setRequestProperty("Charset", "UTF-8");

            // 设置文件类型:
            conn.setRequestProperty("Content-Type","application/json; charset=UTF-8");

            // 设置接收类型否则返回415错误
            //conn.setRequestProperty("accept","*/*")此处为暴力方法设置接受所有类型，以此来防范返回415;
            conn.setRequestProperty("accept","application/json");
            conn.setConnectTimeout(5000);
            conn.setReadTimeout(5000);

            String dataStr = sendTemplateData.toString();

            log.warn("发送数据:" + dataStr);
            // 往服务器里面发送数据
            byte[] buffer = dataStr.getBytes("UTF-8");

            // 设置文件长度
            conn.setRequestProperty("Content-Length", String.valueOf(buffer.length));
            OutputStream output = conn.getOutputStream();
            output.write(buffer);
            output.flush();
            output.close();

            if (conn.getResponseCode() == 200) {
                BufferedReader reader = new BufferedReader(new InputStreamReader(conn.getInputStream(),"UTF-8"));

                StringBuilder sb = new StringBuilder();
                String line = "";

                while ((line = reader.readLine()) != null) {
                    sb.append(line);
                }

                result = sb.toString();

                log.warn("接收数据:" + result);

                reader.close();
            }
        } catch (Exception ex) {
            log.error(ex.getMessage(),ex);
        }

        return result;
    }

    public static String getContent(String content) {
        String finallyContent = content;
        String regex = "<p.*?>(.*?)</p>";
        Pattern p = Pattern.compile(regex);
        Matcher m = p.matcher(content);
        while (m.find()) {
            finallyContent = m.group(1);
        }


        return finallyContent;
    }

    /**
     * 微信发送模版
     * @return
     */
    public static boolean sendMessageTemplate(String userName,String content,
                                           String sendRemark,
                                           String sendDate,
                                           String openId, String templateId,String tokenUrl, String appId, String appSecret) {
        JSONObject sendData = new JSONObject();

        JSONObject first = new JSONObject();
        first.put("value", "审核结果通知");
        first.put("color", "#FF0000");

        JSONObject keyword1 = new JSONObject();
        keyword1.put("value", userName);
        keyword1.put("color", "#173177");

        JSONObject keyword2 = new JSONObject();
        keyword2.put("value", content);
        keyword2.put("color", "#173177");

        JSONObject keyword3 = new JSONObject();
        keyword3.put("value", sendDate);
        keyword3.put("color", "#173177");

        JSONObject remark = new JSONObject();
        remark.put("value", sendRemark);
        remark.put("color", "#173177");

        sendData.put("first", first);
        sendData.put("keyword1", keyword1);
        sendData.put("keyword2", keyword2);
        sendData.put("keyword3", keyword3);
        sendData.put("remark", remark);

        //w8Zk_VQMFIEVSIBPZid7zSrvHmBdrgnqF76u8PLCZEs cs
        boolean ret = sendTemplate(sendData,tokenUrl, appId, appSecret, templateId, openId, "http://wsbm.xiaoxinda.com/#/resultQuery");

        return ret;
    }

    /**
     * 微信发送模版
     * 跳转新闻
     * @return
     */
    public static boolean sendNewsMessageTemplate(String userName,String content,
                                               String sendRemark,
                                               String sendDate,
                                               String openId, String templateId,String tokenUrl, String appId, String appSecret,String url) {
        JSONObject sendData = new JSONObject();

        if(com.jpsoft.campus.modules.common.utils.StringUtils.isEmpty(url)){
            url = "http://wsbm.xiaoxinda.com/#/resultQuery";
        }

        JSONObject first = new JSONObject();
        first.put("value", "审核结果通知");
        first.put("color", "#FF0000");

        JSONObject keyword1 = new JSONObject();
        keyword1.put("value", userName);
        keyword1.put("color", "#173177");

        JSONObject keyword2 = new JSONObject();
        keyword2.put("value", content);
        keyword2.put("color", "#173177");

        JSONObject keyword3 = new JSONObject();
        keyword3.put("value", sendDate);
        keyword3.put("color", "#173177");

        JSONObject remark = new JSONObject();
        remark.put("value", sendRemark);
        remark.put("color", "#173177");

        sendData.put("first", first);
        sendData.put("keyword1", keyword1);
        sendData.put("keyword2", keyword2);
        sendData.put("keyword3", keyword3);
        sendData.put("remark", remark);

        //w8Zk_VQMFIEVSIBPZid7zSrvHmBdrgnqF76u8PLCZEs cs
        boolean ret = sendTemplate(sendData,tokenUrl, appId, appSecret, templateId, openId, url);

        return ret;
    }

    public static boolean sendCollectTemplate(String userName,String content,
                                              String sendRemark,
                                              String sendDate,
                                              String openId, String templateId,String tokenUrl, String appId, String appSecret) {
        JSONObject sendData = new JSONObject();

        JSONObject first = new JSONObject();
        first.put("value", "尊敬的家长，您好，根据经开区2022年秋季招生入学工作方案，即将进行C1类生随机派位流程。");
        first.put("color", "#FF0000");

        JSONObject keyword1 = new JSONObject();
        keyword1.put("value", "实验中学东校区C1类生随机派位");
        keyword1.put("color", "#173177");

        JSONObject keyword2 = new JSONObject();
        keyword2.put("value", content);
        keyword2.put("color", "#173177");

//        JSONObject keyword3 = new JSONObject();
//        keyword3.put("value", sendDate);
//        keyword3.put("color", "#173177");
//
        JSONObject remark = new JSONObject();
        remark.put("value", sendRemark);
        remark.put("color", "#173177");

        sendData.put("first", first);
        sendData.put("keyword1", keyword1);
        sendData.put("keyword2", keyword2);
        //sendData.put("keyword3", keyword3);
        sendData.put("remark", remark);

        //w8Zk_VQMFIEVSIBPZid7zSrvHmBdrgnqF76u8PLCZEs cs
        boolean ret = sendTemplate(sendData,tokenUrl, appId, appSecret, templateId, openId, "http://wsbm.xiaoxinda.com/#/other/quest");

        return ret;
    }


    public static void main(String[] args) {
		/*AccessToken at = WeixinUtil.getCommonAccessToken(APP_ID, APP_SECRET,"061iw9gy1sZ1dg09dqhy1ae3gy1iw9gq");

		System.out.println(at.getOpenid());*/

		/*for (int i = 0; i < 1; i++) {
			doorMessage("1111", 2,"2018-09-22 09:00-12:00", 1, "2222222", "3333333", "oiFbN0p9jKphExeGIa3HtyL9HjqM");
		}*/

        //orderConfirmMessage("12312321312", 1111, DEFAULT_OPEN_ID);
        //sendBillInfo("测试", "测sendNoticeMessage试", "荆鹏创业", "张三", new BigDecimal(1), "o64fbt6TTUcqLC_tyVJ60I5kzWVc", "1111", new Date());
        //sendBillPayedMessage("测试", "荆鹏创业", "张三", new BigDecimal(1), "o64fbt6TTUcqLC_tyVJ60I5kzWVc", "11111", new Date());
        //sendScanPayedMessage("测试", "荆鹏创业", "张三11111", new BigDecimal(1), "o64fbt6TTUcqLC_tyVJ60I5kzWVc", "11111", new Date());
//sendNoticeMessage("测试", "sdfsdfasfasdfafasdfasdf", "oHjCawgwCGen5k1-hAsimdEX5lZo", "2222","9JKu9t9uTPHL1siIm7WatOJ1KXGolzEPlnmqIbckTUM","wx41ac6791b9dbcf52","3e3d47c5f132d4b47636a536d503fbd4");

	/*	sendBillInfo("1栋-1单元-101",new BigDecimal("0.01"),"oOlFd1BUR6Fi2ayBWUeou81G8ZD4",
				"物业费","小鹏物业","测试","9e52c923-2179-4526-a108-32becc9aa1d0","vVraOPl4OyNVWDGz6XYlJqRb7qGjjcyJdnVi5HWmFRI","wx41ac6791b9dbcf52","3e3d47c5f132d4b47636a536d503fbd4");*/

        /*sendBillPayedMessage("1栋-1单元-101","物业费",new Date(),new BigDecimal("0.01"),"oOlFd1BUR6Fi2ayBWUeou81G8ZD4","33e7e5e6-5b38-45f5-9ccf-c7ef21978a5b","kbx5u70nzo7PklTVl4B1xvRffksrUtXxnDll8ggEKlo","wx41ac6791b9dbcf52","3e3d47c5f132d4b47636a536d503fbd4");*/

        //sendAlarmNoticeMessage("周界入侵警报", "周界入侵，目标ID：1419，区域范围：(0.327,0.698)(0.674,0.936)(0.67,0.985)(0.352,0.973)", "oHjCawgwCGen5k1-hAsimdEX5lZo", "","8dpP8XM1XaiiIByXiuAeFIZeNjEow1AleG_h8O2xyVA","wx7e70eb62a8459869","909d17e353268da57c4f18cc09798049");
        //sendNoticeMessage("测试", "sdfsdfasfasdfafasdfasdf", "oHjCawgwCGen5k1-hAsimdEX5lZo", "2222","vsw6sb9lMA3bcsE8MdeaGENvI1HSUy2ht9B8hS3jEzA","wx7e70eb62a8459869","909d17e353268da57c4f18cc09798049");
//        sendTemperatureAlarmInfo("oLowyuOQ9ULF3LUGt6h3fTrsnbVE", "荆鹏软件_门房考勤机", "wx0b3c41a903053808", "43557bd62f77b0c3d6670e991872f0e7", "Mg9Ldk_kaoHAUwXFHEatrGugTlOz3yrMmMk7VoBca4M");
//        getContent("<p>内容</p>");
    }
}
