package com.jpsoft.campus.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.campus.modules.base.dto.AdjustDetailDTO;
import com.jpsoft.campus.modules.base.entity.AdjustDetail;
import com.jpsoft.campus.modules.base.entity.ApplicationPrimary;
import com.jpsoft.campus.modules.base.service.AdjustDetailService;
import com.jpsoft.campus.modules.base.service.ApplicationPrimaryService;
import com.jpsoft.campus.modules.common.dto.MessageResult;
import com.jpsoft.campus.modules.common.utils.PojoUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.jpsoft.campus.modules.common.dto.Sort;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/adjustDetail")
@Api(description = "adjustDetail")
public class AdjustDetailController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private AdjustDetailService adjustDetailService;
    @Autowired
    private ApplicationPrimaryService applicationPrimaryService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<AdjustDetail> create(){
        MessageResult<AdjustDetail> msgResult = new MessageResult<>();

        AdjustDetail adjustDetail = new AdjustDetail();

        msgResult.setData(adjustDetail);
        msgResult.setResult(true);

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<AdjustDetail> edit(@PathVariable("id") String id){
        MessageResult<AdjustDetail> msgResult = new MessageResult<>();

        try {
            AdjustDetail adjustDetail = adjustDetailService.get(id);

            if (adjustDetail != null) {
                msgResult.setResult(true);
                msgResult.setData(adjustDetail);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<AdjustDetail> update(@RequestBody AdjustDetail adjustDetail,@RequestAttribute String subject){
        MessageResult<AdjustDetail> msgResult = new MessageResult<>();

        try {
            adjustDetail.setUpdateBy(subject);
            adjustDetail.setUpdateTime(new Date());
            
            int affectCount = adjustDetailService.update(adjustDetail);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(adjustDetail);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            AdjustDetail adjustDetail = adjustDetailService.get(id);
            adjustDetail.setDelFlag(true);
            adjustDetail.setUpdateBy(subject);
            adjustDetail.setUpdateTime(new Date());

            int affectCount = adjustDetailService.update(adjustDetail);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                AdjustDetail adjustDetail = adjustDetailService.get(id);
                adjustDetail.setDelFlag(true);
                adjustDetail.setUpdateBy(subject);
                adjustDetail.setUpdateTime(new Date());

                affectCount += adjustDetailService.update(adjustDetail);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="调剂列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String schoolId,String idCardOrName,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(schoolId)) {
            searchParams.put("schoolId",schoolId);
        }
        if (StringUtils.isNotEmpty(idCardOrName)) {
            searchParams.put("idCardOrName",idCardOrName);
        }

        Page<AdjustDetailDTO> page = adjustDetailService.pageSearchDTO(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="调剂学校")
    @PostMapping("add")
    public MessageResult<AdjustDetail> add(String applicationId,String schoolId,@RequestAttribute String subject){
        MessageResult<AdjustDetail> msgResult = new MessageResult<>();

        try {
            ApplicationPrimary applicationPrimary = applicationPrimaryService.get(applicationId);
            applicationPrimary.setOfferId(schoolId);
            applicationPrimary.setStatus(String.valueOf(Integer.parseInt(applicationPrimary.getStatus()) + 10));
            applicationPrimary.setUpdateBy(subject);
            applicationPrimary.setUpdateTime(new Date());
            applicationPrimaryService.update(applicationPrimary);

            //写入调剂
            AdjustDetail adjustDetail = new AdjustDetail();
            adjustDetail.setId(UUID.randomUUID().toString());
            adjustDetail.setApplicationId(applicationPrimary.getId());
            adjustDetail.setStudentId(applicationPrimary.getStudentId());
            adjustDetail.setPersonId(applicationPrimary.getPersonId());
            adjustDetail.setSchoolId(schoolId);
            adjustDetail.setDelFlag(false);
            adjustDetail.setCreateBy(subject);
            adjustDetail.setCreateTime(new Date());
            adjustDetail.setUpdateBy(subject);
            adjustDetail.setUpdateTime(new Date());
            int affectCount = adjustDetailService.update(adjustDetail);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(adjustDetail);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }
}
