package com.jpsoft.campus.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.campus.modules.base.dto.ApplicationMiddleDTO;
import com.jpsoft.campus.modules.base.dto.ApplicationPrimaryDTO;
import com.jpsoft.campus.modules.base.dto.AttachmentDTO;
import com.jpsoft.campus.modules.base.entity.*;
import com.jpsoft.campus.modules.base.service.*;
import com.jpsoft.campus.modules.common.dto.Sort;
import com.jpsoft.campus.modules.common.dto.MessageResult;
import com.jpsoft.campus.modules.common.utils.PojoUtils;
import com.jpsoft.campus.modules.sys.entity.DataDictionary;
import com.jpsoft.campus.modules.sys.entity.User;
import com.jpsoft.campus.modules.sys.service.DataDictionaryService;
import com.jpsoft.campus.modules.sys.service.UserRoleService;
import com.jpsoft.campus.modules.sys.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/apportion")
@Api(description = "apportion")
public class ApportionController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private ApportionService apportionService;
    @Autowired
    private ApportionDetailService apportionDetailService;
    @Autowired
    private SchoolInfoService schoolInfoService;
    @Autowired
    private DataDictionaryService dataDictionaryService;
    @Autowired
    private UserService userService;
    @Autowired
    private ApplicationPrimaryService applicationPrimaryService;
    @Autowired
    private ApplicationMiddleService applicationMiddleService;
    @Autowired
    private UserRoleService userRoleService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<Apportion> create(){
        MessageResult<Apportion> msgResult = new MessageResult<>();

        Apportion apportion = new Apportion();

        msgResult.setData(apportion);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<Apportion> add(@RequestBody Apportion apportion,@RequestAttribute String subject){
        MessageResult<Apportion> msgResult = new MessageResult<>();

        try {
            apportion.setId(UUID.randomUUID().toString());
            apportion.setDelFlag(false);
            apportion.setCreateBy(subject);
            apportion.setCreateTime(new Date());
            
            int affectCount = apportionService.insert(apportion);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(apportion);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<Apportion> edit(@PathVariable("id") String id){
        MessageResult<Apportion> msgResult = new MessageResult<>();

        try {
            Apportion apportion = apportionService.get(id);

            if (apportion != null) {
                msgResult.setResult(true);
                msgResult.setData(apportion);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<Apportion> update(@RequestBody Apportion apportion,@RequestAttribute String subject){
        MessageResult<Apportion> msgResult = new MessageResult<>();

        try {
            apportion.setUpdateBy(subject);
            apportion.setUpdateTime(new Date());
            
            int affectCount = apportionService.update(apportion);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(apportion);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            Apportion apportion = apportionService.get(id);
            apportion.setDelFlag(true);
            apportion.setUpdateBy(subject);
            apportion.setUpdateTime(new Date());

            int affectCount = apportionService.update(apportion);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                Apportion apportion = apportionService.get(id);
                apportion.setDelFlag(true);
                apportion.setUpdateBy(subject);
                apportion.setUpdateTime(new Date());

                affectCount += apportionService.update(apportion);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String schoolId,String type,String startDate,String endDate,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("create_time","desc"));

        if (StringUtils.isNotEmpty(schoolId)) {
            searchParams.put("schoolId",schoolId);
        }
        if (StringUtils.isNotEmpty(type)) {
            searchParams.put("type",type);
        }
        if (StringUtils.isNotEmpty(startDate)) {
            searchParams.put("startDate",startDate);
        }
        if (StringUtils.isNotEmpty(endDate)) {
            try {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                Date date = sdf.parse(endDate);

                Calendar calendar = Calendar.getInstance();
                calendar.setTime(date);
                calendar.add(calendar.DATE, 1);
                searchParams.put("endDate", sdf.format(calendar.getTime()));
            }
            catch (ParseException e){
                e.printStackTrace();
            }
        }

        Page<Apportion> page = apportionService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);
        for(Apportion apportion : page.getResult()){
            SchoolInfo schoolInfo = schoolInfoService.get(apportion.getSchoolId());
            apportion.setSchoolName(schoolInfo.getName());
            DataDictionary dataDictionary = dataDictionaryService.get(apportion.getType());
            apportion.setTypeName(dataDictionary.getName());
            User user = userService.get(apportion.getCreateBy());
            apportion.setCreateByName(user.getRealName());
        }

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="开始派位-小学")
    @PostMapping("startApportionPrimary")
    public MessageResult<Map> startApportionPrimary(String schoolId,
                                                    @RequestParam(value="grade",defaultValue="") String grade,
                                                    String type,Integer amount,@RequestAttribute String subject){
        MessageResult<Map> msgResult = new MessageResult<>();
        Map<String,Object> map = new HashMap<>();

        try {
            //读取等待派位人员信息列表
            Map<String,Object> bzSearchParams = new HashMap<>();
            Map<String,Object> searchParams = new HashMap<>();
            searchParams.put("status","60");
            bzSearchParams.put("status","60");
            if (StringUtils.isNotEmpty(schoolId)) {
                searchParams.put("schoolId",schoolId);
                bzSearchParams.put("schoolId",schoolId);
            }
            if (StringUtils.isNotEmpty(grade)) {
                searchParams.put("grade",grade);
                bzSearchParams.put("grade",grade);
            }
            if (StringUtils.isNotEmpty(type)) {
                DataDictionary dataDictionary = dataDictionaryService.findByCatalogNameAndValue("C类生证明",type);
                searchParams.put("categoryId",dataDictionary.getId());
                bzSearchParams.put("categoryId",dataDictionary.getId());
            }
            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("a.create_time","desc"));

            //必中
            bzSearchParams.put("hit","1");
            bzSearchParams.put("delFlag","0");
            Page<ApplicationPrimary> bzPage = applicationPrimaryService.pageSearch(bzSearchParams,1,1000000,false,sortList);

            if(bzPage.size() == 0 ){
                bzSearchParams.put("hit","2");
                bzSearchParams.put("delFlag","0");
                bzPage = applicationPrimaryService.pageSearch(bzSearchParams,1,100000,false,sortList);
            }

            //随机
            searchParams.put("notHit","0");
            searchParams.put("delFlag","0");
            Page<ApplicationPrimary> applicationPrimaryPage = applicationPrimaryService.pageSearch(searchParams,1,100000,false,sortList);

            //学位数-必中的人数=剩余的数量
            Integer syAmount = amount - bzPage.size();
            if(syAmount < 0){
                syAmount = 0;
            }

            //随机抽取
            if(amount > applicationPrimaryPage.size()){
                amount = applicationPrimaryPage.size();
            }

            List<Integer> integerList = new ArrayList<>();
//            Random random = new Random();
//            for(int i=0;i<syAmount;i++){
//                for(int j=0;j>-1;j++) {
//                    Integer ran = random.nextInt(applicationPrimaryPage.size());
//                    if (!integerList.contains(ran)) {
//                        integerList.add(ran);
//                        break;
//                    }
//                }
//            }

            //写入派位信息
            Apportion apportion = new Apportion();
            apportion.setId(UUID.randomUUID().toString());
            apportion.setSchoolId(schoolId);
            apportion.setType(type);
            apportion.setDelFlag(false);
            apportion.setCreateBy(subject);
            apportion.setCreateTime(new Date());
            apportion.setUpdateBy(subject);
            apportion.setUpdateTime(new Date());
            apportionService.insert(apportion);

            if(amount < 1){
                map.put("apportionId",apportion.getId());
                msgResult.setResult(true);
                msgResult.setData(map);
                return msgResult;
            }

            //必中
            for(int i=0;i<bzPage.size();i++){
                ApplicationPrimary applicationPrimary = bzPage.get(i);
                ApportionDetail apportionDetail = new ApportionDetail();
                apportionDetail.setId(UUID.randomUUID().toString());
                apportionDetail.setApportionId(apportion.getId());
                apportionDetail.setApplicationId(applicationPrimary.getId());
                apportionDetail.setStudentId(applicationPrimary.getStudentId());
                apportionDetail.setPersonId(applicationPrimary.getPersonId());
                apportionDetail.setSchoolId(schoolId);
                apportionDetail.setWinner(true);
                apportionDetail.setDelFlag(false);
                apportionDetail.setCreateBy(subject);
                apportionDetail.setCreateTime(new Date());
                apportionDetail.setUpdateBy(subject);
                apportionDetail.setUpdateTime(new Date());
                apportionDetailService.insert(apportionDetail);

                applicationPrimary.setStatus(String.valueOf(Integer.parseInt(applicationPrimary.getStatus()) + 10));
                applicationPrimary.setOfferId(schoolId);
                applicationPrimary.setUpdateBy(subject);
                applicationPrimary.setUpdateTime(new Date());
                applicationPrimaryService.update(applicationPrimary);
            }

            //写入派位列表
            for(int i=0;i<applicationPrimaryPage.size();i++){
                ApplicationPrimary applicationPrimary = applicationPrimaryPage.get(i);
                ApportionDetail apportionDetail = new ApportionDetail();
                apportionDetail.setId(UUID.randomUUID().toString());
                apportionDetail.setApportionId(apportion.getId());
                apportionDetail.setApplicationId(applicationPrimary.getId());
                apportionDetail.setStudentId(applicationPrimary.getStudentId());
                apportionDetail.setPersonId(applicationPrimary.getPersonId());
                apportionDetail.setSchoolId(schoolId);
//                Boolean bl = integerList.contains(i) == true ? true : false;
                Boolean bl = true;
                apportionDetail.setWinner(bl);
                apportionDetail.setDelFlag(false);
                apportionDetail.setCreateBy(subject);
                apportionDetail.setCreateTime(new Date());
                apportionDetail.setUpdateBy(subject);
                apportionDetail.setUpdateTime(new Date());
                apportionDetailService.insert(apportionDetail);

                if(bl){
                    applicationPrimary.setStatus(String.valueOf(Integer.parseInt(applicationPrimary.getStatus()) + 10));
                    applicationPrimary.setOfferId(schoolId);
                    applicationPrimary.setUpdateBy(subject);
                    applicationPrimary.setUpdateTime(new Date());
                    applicationPrimaryService.update(applicationPrimary);
                }
            }

            map.put("apportionId",apportion.getId());
            msgResult.setResult(true);
            msgResult.setData(map);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="开始派位-小学-测试")
    @PostMapping("startApportionPrimaryTest")
    public MessageResult<Map> startApportionPrimaryTest(String schoolId,
                                                        @RequestParam(value="grade",defaultValue="") String grade,
                                                        String type,Integer amount,@RequestAttribute String subject){
        MessageResult<Map> msgResult = new MessageResult<>();
        Map<String,Object> map = new HashMap<>();

        try {
            //读取等待派位人员信息列表
            Map<String,Object> searchParams = new HashMap<>();
            searchParams.put("status","60");
            if (StringUtils.isNotEmpty(schoolId)) {
                searchParams.put("schoolId",schoolId);
            }
            if (StringUtils.isNotEmpty(grade)) {
                searchParams.put("grade",grade);
            }
            if (StringUtils.isNotEmpty(type)) {
                DataDictionary dataDictionary = dataDictionaryService.findByCatalogNameAndValue("C类生证明",type);
                searchParams.put("categoryId",dataDictionary.getId());
            }
            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time","desc"));
            Page<ApplicationPrimary> applicationPrimaryPage = applicationPrimaryService.pageSearch(searchParams,1,100000,false,sortList);

            //随机抽取
            if(amount > applicationPrimaryPage.size()){
                amount = applicationPrimaryPage.size();
            }
            List<Integer> integerList = new ArrayList<>();
            Random random = new Random();
            for(int i=0;i<amount;i++){
                for(int j=0;j>-1;j++) {
                    Integer ran = random.nextInt(applicationPrimaryPage.size());
                    if (!integerList.contains(ran)) {
                        integerList.add(ran);
                        break;
                    }
                }
            }

            //写入派位信息
            Apportion apportion = new Apportion();
            apportion.setId(UUID.randomUUID().toString());
            apportion.setSchoolId(schoolId);
            apportion.setType(type);
            apportion.setDelFlag(false);
            apportion.setCreateBy("测试派位系统");
            apportion.setCreateTime(new Date());
            apportionService.insert(apportion);

            if(amount < 1){
                map.put("apportionId",apportion.getId());
                msgResult.setResult(true);
                msgResult.setData(map);
                return msgResult;
            }

            //写入派位列表
            for(int i=0;i<applicationPrimaryPage.size();i++){
                ApplicationPrimary applicationPrimary = applicationPrimaryPage.get(i);
                ApportionDetail apportionDetail = new ApportionDetail();
                apportionDetail.setId(UUID.randomUUID().toString());
                apportionDetail.setApportionId(apportion.getId());
                apportionDetail.setApplicationId(applicationPrimary.getId());
                apportionDetail.setStudentId(applicationPrimary.getStudentId());
                apportionDetail.setPersonId(applicationPrimary.getPersonId());
                apportionDetail.setSchoolId(schoolId);
                Boolean bl = integerList.contains(i) == true ? true : false;
                apportionDetail.setWinner(bl);
                apportionDetail.setDelFlag(true);
                apportionDetail.setCreateBy("测试派位系统");
                apportionDetail.setCreateTime(new Date());
                apportionDetailService.insert(apportionDetail);
            }

            map.put("apportionId",apportion.getId());
            msgResult.setResult(true);
            msgResult.setData(map);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageListPrimary",method = RequestMethod.POST)
    public MessageResult<Map> pageListPrimary(
            @RequestParam(value="schoolId",defaultValue="") String schoolId,
            @RequestParam(value="grade",defaultValue="") String grade,
            @RequestParam(value="type",defaultValue="") String type,
            @RequestParam(value="types",defaultValue="") String types,
            @RequestParam(value="categoryValue",defaultValue="") String categoryValue,
            @RequestParam(value="status",defaultValue="") String status,
            @RequestParam(value="categoryId",defaultValue="") String categoryId,
            @RequestParam(value="delFlag",defaultValue="false") Boolean delFlag,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestParam(value="startStatus",defaultValue="") String startStatus,
            @RequestParam(value="endStatus",defaultValue="") String endStatus,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.create_time","desc"));

        if (StringUtils.isNotEmpty(status)) {
            searchParams.put("status",status);
        }

        //开始
        if (StringUtils.isNotEmpty(startStatus)) {
            searchParams.put("startStatus",startStatus);
        }
        //结束
        if (StringUtils.isNotEmpty(endStatus)) {
            searchParams.put("endStatus",endStatus);
        }

        if(StringUtils.isNotEmpty(type)){
            searchParams.put("type",type);
        }

        if(StringUtils.isNotEmpty(types)){
            String[] typeList = types.split(",");
            searchParams.put("typeList",typeList);
        }

        if (StringUtils.isNotEmpty(categoryValue)) {
            DataDictionary dataDictionary = dataDictionaryService.findByCatalogNameAndValue("C类生证明",categoryValue);
            searchParams.put("categoryId",dataDictionary.getId());
        }

        searchParams.put("delFlag",delFlag);

        if(StringUtils.isNotEmpty(schoolId)){
            searchParams.put("code", "%" + schoolId + "%");
        }else{
            if (!userService.hasRole(subject,"SYSADMIN")) {
                User user = userService.get(subject);
                searchParams.put("code", "%" + user.getSchoolId() + "%");
            }
        }

        if (StringUtils.isNotEmpty(grade)) {
            searchParams.put("grade",grade);
        }


        //根据用户和角色查询
        int count1 = userRoleService.findCountByAIdAndRName(subject,"SYSADMIN");
        int count2 = userRoleService.findCountByAIdAndRName(subject,"XFLD");
        int count3 = userRoleService.findCountByAIdAndRName(subject,"XFGZRY");

        if(count1 > 0){
            //管理员查全部
        }else if(count2 > 0){
            //领导查全部
        }else if(count3 > 0){
            //工作人员只能查网上初审10和现场初审20
            List statusList = new ArrayList();
            statusList.add("10");
            statusList.add("20");
            searchParams.put("orStatus",statusList);
        }

        Page<ApplicationPrimaryDTO> page = applicationPrimaryService.pageSearchDTO(searchParams,pageIndex,pageSize,true,sortList);
        for(ApplicationPrimaryDTO applicationPrimaryDTO : page.getResult()){
            List<ApportionDetail> adList = apportionDetailService.findByStudentId(applicationPrimaryDTO.getStudentId());
            ApportionDetail apportionDetail = new ApportionDetail();
            if(adList.size() > 0){
                apportionDetail = adList.get(0);
                SchoolInfo school = schoolInfoService.get(apportionDetail.getSchoolId());
                apportionDetail.setSchoolName(school.getName());
            }
            applicationPrimaryDTO.setApportionDetail(apportionDetail);
            applicationPrimaryDTO.setCardType(dataDictionaryService.findNameByCatalogNameAndValue("证件类型",applicationPrimaryDTO.getCardType()));
            applicationPrimaryDTO.setCategoryName(dataDictionaryService.getValue(applicationPrimaryDTO.getCategoryId()));
            applicationPrimaryDTO.setStatusName(dataDictionaryService.findNameByCatalogNameAndValue("小学招生审核流程",applicationPrimaryDTO.getStatus()));
        }

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }



    @ApiOperation(value="开始派位-中学")
    @PostMapping("startApportionMiddle")
    public MessageResult<Map> startApportionMiddle(String schoolId,
                                                   @RequestParam(value="grade",defaultValue="") String grade,
                                                   String type,Integer amount,@RequestAttribute String subject){
        MessageResult<Map> msgResult = new MessageResult<>();
        Map<String,Object> map = new HashMap<>();

        try {
            //读取等待派位人员信息列表
            Map<String,Object> bzSearchParams = new HashMap<>();
            Map<String,Object> searchParams = new HashMap<>();
            bzSearchParams.put("status","60");
            searchParams.put("status","60");
            if (StringUtils.isNotEmpty(schoolId)) {
                bzSearchParams.put("schoolId",schoolId);
                searchParams.put("schoolId",schoolId);
            }
            if (StringUtils.isNotEmpty(grade)) {
                searchParams.put("grade",grade);
            }
            if (StringUtils.isNotEmpty(type)) {
                bzSearchParams.put("type",type);
                searchParams.put("type",type);
            }
            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time","desc"));

            //必中
            bzSearchParams.put("hit","1");
            bzSearchParams.put("delFlag","0");
            Page<ApplicationMiddle> bzPage = applicationMiddleService.pageSearch(bzSearchParams,1,100000,false,sortList);
            //随机
            searchParams.put("hit","0");
            searchParams.put("delFlag","0");
            Page<ApplicationMiddle> ApplicationMiddlePage = applicationMiddleService.pageSearch(searchParams,1,100000,false,sortList);

            //学位数-必中的人数=剩余的数量
            Integer syAmount = amount - bzPage.size();
            if(syAmount < 0){
                syAmount = 0;
            }

            //随机抽取
            if(syAmount > ApplicationMiddlePage.size()){
                 syAmount = ApplicationMiddlePage.size();
            }
            List<Integer> integerList = new ArrayList<>();
            Random random = new Random();
            for(int i=0;i<syAmount;i++){
                for(int j=0;j>-1;j++) {
                    Integer ran = random.nextInt(ApplicationMiddlePage.size());
                    if (!integerList.contains(ran)) {
                        integerList.add(ran);
                        break;
                    }
                }
            }

            //写入派位信息
            Apportion apportion = new Apportion();
            apportion.setId(UUID.randomUUID().toString());
            apportion.setSchoolId(schoolId);
            apportion.setType(type);
            apportion.setDelFlag(false);
            apportion.setCreateBy(subject);
            apportion.setCreateTime(new Date());
            apportion.setUpdateBy(subject);
            apportion.setUpdateTime(new Date());
            apportionService.insert(apportion);

            if(amount < 1){
                map.put("apportionId",apportion.getId());
                msgResult.setResult(true);
                msgResult.setData(map);
                return msgResult;
            }

            //必中
            for(int i=0;i<bzPage.size();i++){
                ApplicationMiddle applicationMiddle = bzPage.get(i);
                ApportionDetail apportionDetail = new ApportionDetail();
                apportionDetail.setId(UUID.randomUUID().toString());
                apportionDetail.setApportionId(apportion.getId());
                apportionDetail.setApplicationId(applicationMiddle.getId());
                apportionDetail.setStudentId(applicationMiddle.getStudentId());
                apportionDetail.setPersonId(applicationMiddle.getPersonId());
                apportionDetail.setSchoolId(schoolId);
                apportionDetail.setWinner(true);
                apportionDetail.setDelFlag(false);
                apportionDetail.setCreateBy(subject);
                apportionDetail.setCreateTime(new Date());
                apportionDetail.setUpdateBy(subject);
                apportionDetail.setUpdateTime(new Date());
                apportionDetailService.insert(apportionDetail);

                applicationMiddle.setStatus(String.valueOf(Integer.parseInt(applicationMiddle.getStatus()) + 10));
                applicationMiddle.setOfferId(schoolId);
                applicationMiddle.setUpdateBy(subject);
                applicationMiddle.setUpdateTime(new Date());
                applicationMiddleService.update(applicationMiddle);
            }

            //写入派位列表
            for(int i=0;i<ApplicationMiddlePage.size();i++){
                ApplicationMiddle applicationMiddle = ApplicationMiddlePage.get(i);
                ApportionDetail apportionDetail = new ApportionDetail();
                apportionDetail.setId(UUID.randomUUID().toString());
                apportionDetail.setApportionId(apportion.getId());
                apportionDetail.setApplicationId(applicationMiddle.getId());
                apportionDetail.setStudentId(applicationMiddle.getStudentId());
                apportionDetail.setPersonId(applicationMiddle.getPersonId());
                apportionDetail.setSchoolId(schoolId);
                Boolean bl = integerList.contains(i) == true ? true : false;
                apportionDetail.setWinner(bl);
                apportionDetail.setDelFlag(false);
                apportionDetail.setCreateBy(subject);
                apportionDetail.setCreateTime(new Date());
                apportionDetail.setUpdateBy(subject);
                apportionDetail.setUpdateTime(new Date());
                apportionDetailService.insert(apportionDetail);

                if(bl){
                    applicationMiddle.setStatus(String.valueOf(Integer.parseInt(applicationMiddle.getStatus()) + 10));
                    applicationMiddle.setOfferId(schoolId);
                    applicationMiddle.setUpdateBy(subject);
                    applicationMiddle.setUpdateTime(new Date());
                    applicationMiddleService.update(applicationMiddle);
                }
            }

            map.put("apportionId",apportion.getId());
            msgResult.setResult(true);
            msgResult.setData(map);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="开始派位-中学-测试")
    @PostMapping("startApportionMiddleTest")
    public MessageResult<Map> startApportionMiddleTest(String schoolId,
                                                       @RequestParam(value="grade",defaultValue="") String grade,
                                                       String type,Integer amount,@RequestAttribute String subject){
        MessageResult<Map> msgResult = new MessageResult<>();
        Map<String,Object> map = new HashMap<>();

        try {
            //读取等待派位人员信息列表
            Map<String,Object> searchParams = new HashMap<>();
            searchParams.put("status","60");
            if (StringUtils.isNotEmpty(schoolId)) {
                searchParams.put("schoolId",schoolId);
            }
            if (StringUtils.isNotEmpty(grade)) {
                searchParams.put("grade",grade);
            }
            if (StringUtils.isNotEmpty(type)) {
                searchParams.put("type",type);
            }
            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("a.create_time","desc"));
            Page<ApplicationMiddle> ApplicationMiddlePage = applicationMiddleService.pageSearch(searchParams,1,100000,false,sortList);

            //随机抽取
            if(amount > ApplicationMiddlePage.size()){
                amount = ApplicationMiddlePage.size();
            }
            List<Integer> integerList = new ArrayList<>();
            Random random = new Random();
            for(int i=0;i<amount;i++){
                for(int j=0;j>-1;j++) {
                    Integer ran = random.nextInt(ApplicationMiddlePage.size());
                    if (!integerList.contains(ran)) {
                        integerList.add(ran);
                        break;
                    }
                }
            }

            //写入派位信息
            Apportion apportion = new Apportion();
            apportion.setId(UUID.randomUUID().toString());
            apportion.setSchoolId(schoolId);
            apportion.setType(type);
            apportion.setDelFlag(false);
            apportion.setCreateBy("测试派位系统");
            apportion.setCreateTime(new Date());
            apportionService.insert(apportion);

            if(amount < 1){
                map.put("apportionId",apportion.getId());
                msgResult.setResult(true);
                msgResult.setData(map);
                return msgResult;
            }

            //写入派位列表
            for(int i=0;i<ApplicationMiddlePage.size();i++){
                ApplicationMiddle applicationMiddle = ApplicationMiddlePage.get(i);
                ApportionDetail apportionDetail = new ApportionDetail();
                apportionDetail.setId(UUID.randomUUID().toString());
                apportionDetail.setApportionId(apportion.getId());
                apportionDetail.setApplicationId(applicationMiddle.getId());
                apportionDetail.setStudentId(applicationMiddle.getStudentId());
                apportionDetail.setPersonId(applicationMiddle.getPersonId());
                apportionDetail.setSchoolId(schoolId);
                Boolean bl = integerList.contains(i) == true ? true : false;
                apportionDetail.setWinner(bl);
                apportionDetail.setDelFlag(true);
                apportionDetail.setCreateBy("测试派位系统");
                apportionDetail.setCreateTime(new Date());
                apportionDetailService.insert(apportionDetail);
            }

            map.put("apportionId",apportion.getId());
            msgResult.setResult(true);
            msgResult.setData(map);
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="中学列表")
    @RequestMapping(value = "pageListMiddle",method = RequestMethod.POST)
    public MessageResult<Map> pageListMiddle(
            @RequestParam(value="schoolId",defaultValue="") String schoolId,
            @RequestParam(value="grade",defaultValue="") String grade,
            @RequestParam(value="type",defaultValue="") String type,
            @RequestParam(value="categoryValue",defaultValue="") String categoryValue,
            @RequestParam(value="status",defaultValue="") String status,
            @RequestParam(value="categoryId",defaultValue="") String categoryId,
            @RequestParam(value="delFlag",defaultValue="false") Boolean delFlag,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestParam(value="startStatus",defaultValue="") String startStatus,
            @RequestParam(value="endStatus",defaultValue="") String endStatus,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.create_time","desc"));

        if (StringUtils.isNotEmpty(status)) {
            searchParams.put("status",status);
        }

        //开始
        if (StringUtils.isNotEmpty(startStatus)) {
            searchParams.put("startStatus",startStatus);
        }
        //结束
        if (StringUtils.isNotEmpty(endStatus)) {
            searchParams.put("endStatus",endStatus);
        }

        if(StringUtils.isNotEmpty(type)){
            searchParams.put("type",type);
        }

        if (StringUtils.isNotEmpty(categoryValue)) {
            searchParams.put("categoryId",categoryValue);
        }

        searchParams.put("delFlag",delFlag);

        if(StringUtils.isNotEmpty(schoolId)){
            searchParams.put("code", "%" + schoolId + "%");
        }else{
            if (!userService.hasRole(subject,"SYSADMIN")) {
                User user = userService.get(subject);
                searchParams.put("code", "%" + user.getSchoolId() + "%");
            }
        }

        if (StringUtils.isNotEmpty(grade)) {
            searchParams.put("grade",grade);
        }


        //根据用户和角色查询
        int count1 = userRoleService.findCountByAIdAndRName(subject,"SYSADMIN");
        int count2 = userRoleService.findCountByAIdAndRName(subject,"XFLD");
        int count3 = userRoleService.findCountByAIdAndRName(subject,"XFGZRY");

        if(count1 > 0){
            //管理员查全部
        }else if(count2 > 0){
            //领导查全部
        }else if(count3 > 0){
            //工作人员只能查网上初审10和现场初审20
            List statusList = new ArrayList();
            statusList.add("10");
            statusList.add("20");
            searchParams.put("orStatus",statusList);
        }

        Page<ApplicationMiddleDTO> page = applicationMiddleService.pageSearchDTO(searchParams,pageIndex,pageSize,true,sortList);
        for(ApplicationMiddleDTO applicationMiddleDTO : page.getResult()){
            List<ApportionDetail> adList = apportionDetailService.findByStudentId(applicationMiddleDTO.getStudentId());
            ApportionDetail apportionDetail = new ApportionDetail();
            if(adList.size() > 0){
                apportionDetail = adList.get(0);
                SchoolInfo school = schoolInfoService.get(apportionDetail.getSchoolId());
                apportionDetail.setSchoolName(school.getName());
            }
            applicationMiddleDTO.setApportionDetail(apportionDetail);
            applicationMiddleDTO.setCardType(dataDictionaryService.findNameByCatalogNameAndValue("证件类型",applicationMiddleDTO.getCardType()));
            applicationMiddleDTO.setCategoryName(dataDictionaryService.getValue(applicationMiddleDTO.getCategoryId()));
            applicationMiddleDTO.setStatusName(dataDictionaryService.findNameByCatalogNameAndValue("小学招生审核流程",applicationMiddleDTO.getStatus()));
            applicationMiddleDTO.setCategoryName(dataDictionaryService.getName(applicationMiddleDTO.getCategoryId()));
        }

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
