package com.jpsoft.campus.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.campus.modules.base.entity.DistrictInfo;
import com.jpsoft.campus.modules.base.entity.SchoolInfo;
import com.jpsoft.campus.modules.base.service.DistrictInfoService;
import com.jpsoft.campus.modules.base.service.SchoolInfoService;
import com.jpsoft.campus.modules.common.dto.Sort;
import com.jpsoft.campus.modules.common.dto.MessageResult;
import com.jpsoft.campus.modules.common.utils.PojoUtils;
import com.jpsoft.campus.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/districtInfo")
@Api(description = "districtInfo")
public class DistrictInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private DistrictInfoService districtInfoService;
    @Autowired
    private DataDictionaryService dataDictionaryService;

    @Autowired
    private SchoolInfoService schoolInfoService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<DistrictInfo> create(){
        MessageResult<DistrictInfo> msgResult = new MessageResult<>();

        DistrictInfo districtInfo = new DistrictInfo();

        msgResult.setData(districtInfo);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<DistrictInfo> add(@RequestBody DistrictInfo districtInfo,@RequestAttribute String subject){
        MessageResult<DistrictInfo> msgResult = new MessageResult<>();

        try {
            districtInfo.setId(UUID.randomUUID().toString());
            districtInfo.setDelFlag(false);
            districtInfo.setCreateBy(subject);
            districtInfo.setCreateTime(new Date());
            
            int affectCount = districtInfoService.insert(districtInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(districtInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<DistrictInfo> edit(@PathVariable("id") String id){
        MessageResult<DistrictInfo> msgResult = new MessageResult<>();

        try {
            DistrictInfo districtInfo = districtInfoService.get(id);

            if (districtInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(districtInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<DistrictInfo> update(@RequestBody DistrictInfo districtInfo,@RequestAttribute String subject){
        MessageResult<DistrictInfo> msgResult = new MessageResult<>();

        try {
            districtInfo.setUpdateBy(subject);
            districtInfo.setUpdateTime(new Date());
            
            int affectCount = districtInfoService.update(districtInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(districtInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            DistrictInfo districtInfo = districtInfoService.get(id);
            districtInfo.setDelFlag(true);
            districtInfo.setUpdateBy(subject);
            districtInfo.setUpdateTime(new Date());

            int affectCount = districtInfoService.update(districtInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                DistrictInfo districtInfo = districtInfoService.get(id);
                districtInfo.setDelFlag(true);
                districtInfo.setUpdateBy(subject);
                districtInfo.setUpdateTime(new Date());

                affectCount += districtInfoService.update(districtInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="name",defaultValue="") String name,
            @RequestParam(value="streetId",defaultValue="") String streetId,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.create_time","desc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name" , "%" + name + "%");
        }

        if (StringUtils.isNotEmpty(streetId)) {
            searchParams.put("streetId" , streetId);
        }

        Page<DistrictInfo> page = districtInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);
        for(DistrictInfo districtInfo : page.getResult()){
            if(StringUtils.isNotEmpty(districtInfo.getStreetId())) {
                String streeName = dataDictionaryService.getName(districtInfo.getStreetId());
                districtInfo.setStreetName(streeName);
            }

            String[] schoolIds = districtInfo.getSchoolId().split(",");
            StringJoiner schoolNames = new StringJoiner(",");

            for (String schoolId : schoolIds) {
                SchoolInfo schoolInfo = schoolInfoService.get(schoolId);
                schoolNames.add(schoolInfo.getName());
            }

            districtInfo.setSchoolName(schoolNames.toString());
        }

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
