package com.jpsoft.campus.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.campus.modules.base.entity.OfferInfo;
import com.jpsoft.campus.modules.base.service.OfferInfoService;
import com.jpsoft.campus.modules.common.dto.MessageResult;
import com.jpsoft.campus.modules.common.utils.PojoUtils;
import com.jpsoft.campus.modules.common.dto.Sort;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/offerInfo")
@Api(description = "offerInfo")
public class OfferInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private OfferInfoService offerInfoService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<OfferInfo> create(){
        MessageResult<OfferInfo> msgResult = new MessageResult<>();

        OfferInfo offerInfo = new OfferInfo();

        msgResult.setData(offerInfo);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<OfferInfo> add(@RequestBody OfferInfo offerInfo,@RequestAttribute String subject){
        MessageResult<OfferInfo> msgResult = new MessageResult<>();

        try {
            offerInfo.setId(UUID.randomUUID().toString());
            offerInfo.setDelFlag(false);
            offerInfo.setCreateBy(subject);
            offerInfo.setCreateTime(new Date());
            
            int affectCount = offerInfoService.insert(offerInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(offerInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<OfferInfo> edit(@PathVariable("id") String id){
        MessageResult<OfferInfo> msgResult = new MessageResult<>();

        try {
            OfferInfo offerInfo = offerInfoService.get(id);

            if (offerInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(offerInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<OfferInfo> update(@RequestBody OfferInfo offerInfo,@RequestAttribute String subject){
        MessageResult<OfferInfo> msgResult = new MessageResult<>();

        try {
            offerInfo.setUpdateBy(subject);
            offerInfo.setUpdateTime(new Date());
            
            int affectCount = offerInfoService.update(offerInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(offerInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            OfferInfo offerInfo = offerInfoService.get(id);
            offerInfo.setDelFlag(true);
            offerInfo.setUpdateBy(subject);
            offerInfo.setUpdateTime(new Date());

            int affectCount = offerInfoService.update(offerInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                OfferInfo offerInfo = offerInfoService.get(id);
                offerInfo.setDelFlag(true);
                offerInfo.setUpdateBy(subject);
                offerInfo.setUpdateTime(new Date());

                affectCount += offerInfoService.update(offerInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<OfferInfo> page = offerInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
