package com.jpsoft.campus.modules.base.controller;

import com.github.pagehelper.Page;
import com.jpsoft.campus.modules.base.entity.PersonStudent;
import com.jpsoft.campus.modules.base.service.PersonStudentService;
import com.jpsoft.campus.modules.common.dto.Sort;
import com.jpsoft.campus.modules.common.dto.MessageResult;
import com.jpsoft.campus.modules.common.utils.PojoUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/personStudent")
@Api(description = "personStudent")
public class PersonStudentController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private PersonStudentService personStudentService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<PersonStudent> create(){
        MessageResult<PersonStudent> msgResult = new MessageResult<>();

        PersonStudent personStudent = new PersonStudent();

        msgResult.setData(personStudent);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<PersonStudent> add(@RequestBody PersonStudent personStudent,@RequestAttribute String subject){
        MessageResult<PersonStudent> msgResult = new MessageResult<>();

        try {
            personStudent.setId(UUID.randomUUID().toString());
            personStudent.setDelFlag(false);
            personStudent.setCreateBy(subject);
            personStudent.setCreateTime(new Date());
            
            int affectCount = personStudentService.insert(personStudent);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personStudent);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<PersonStudent> edit(@PathVariable("id") String id){
        MessageResult<PersonStudent> msgResult = new MessageResult<>();

        try {
            PersonStudent personStudent = personStudentService.get(id);

            if (personStudent != null) {
                msgResult.setResult(true);
                msgResult.setData(personStudent);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<PersonStudent> update(@RequestBody PersonStudent personStudent,@RequestAttribute String subject){
        MessageResult<PersonStudent> msgResult = new MessageResult<>();

        try {
            personStudent.setUpdateBy(subject);
            personStudent.setUpdateTime(new Date());
            
            int affectCount = personStudentService.update(personStudent);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(personStudent);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            PersonStudent personStudent = personStudentService.get(id);
            personStudent.setDelFlag(true);
            personStudent.setUpdateBy(subject);
            personStudent.setUpdateTime(new Date());

            int affectCount = personStudentService.update(personStudent);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                PersonStudent personStudent = personStudentService.get(id);
                personStudent.setDelFlag(true);
                personStudent.setUpdateBy(subject);
                personStudent.setUpdateTime(new Date());

                affectCount += personStudentService.update(personStudent);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("id_","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        Page<PersonStudent> page = personStudentService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }
}
