package com.jpsoft.campus.modules.base.controller;

import com.alipay.api.domain.CompanyInfo;
import com.github.pagehelper.Page;
import com.jpsoft.campus.modules.base.entity.SchoolInfo;
import com.jpsoft.campus.modules.base.service.SchoolInfoService;
import com.jpsoft.campus.modules.common.dto.Sort;
import com.jpsoft.campus.modules.common.dto.MessageResult;
import com.jpsoft.campus.modules.common.utils.PojoUtils;
import com.jpsoft.campus.modules.sys.entity.User;
import com.jpsoft.campus.modules.sys.service.DataDictionaryService;
import com.jpsoft.campus.modules.sys.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/base/schoolInfo")
@Api(description = "schoolInfo")
public class SchoolInfoController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private SchoolInfoService schoolInfoService;

    @Autowired
    private DataDictionaryService dataDictionaryService;

    @Autowired
    private UserService userService;

    @ApiOperation(value="创建空记录")
    @GetMapping("create")
    public MessageResult<SchoolInfo> create(){
        MessageResult<SchoolInfo> msgResult = new MessageResult<>();

        SchoolInfo schoolInfo = new SchoolInfo();

        msgResult.setData(schoolInfo);
        msgResult.setResult(true);

        return msgResult;
    }
    
    @ApiOperation(value="添加信息")
    @PostMapping("add")
    public MessageResult<SchoolInfo> add(@RequestBody SchoolInfo schoolInfo,@RequestAttribute String subject){
        MessageResult<SchoolInfo> msgResult = new MessageResult<>();

        try {
            schoolInfo.setId(UUID.randomUUID().toString());
            schoolInfo.setDelFlag(false);
            schoolInfo.setCreateBy(subject);
            schoolInfo.setCreateTime(new Date());

            //保存code
            String code = schoolInfo.getId();
            if("1".equals(schoolInfo.getType())){
                //为学校,保存 上级id + , + 自身ID
                code = schoolInfo.getParentId() + "," + code;
            }
            schoolInfo.setCode(code);
            
            int affectCount = schoolInfoService.insert(schoolInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(schoolInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库添加失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="获取信息")
    @GetMapping("edit/{id}")
    public MessageResult<SchoolInfo> edit(@PathVariable("id") String id){
        MessageResult<SchoolInfo> msgResult = new MessageResult<>();

        try {
            SchoolInfo schoolInfo = schoolInfoService.get(id);

            if (schoolInfo != null) {
                msgResult.setResult(true);
                msgResult.setData(schoolInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库不存在该记录!");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="更新用户")
    @PostMapping("update")
    public MessageResult<SchoolInfo> update(@RequestBody SchoolInfo schoolInfo,@RequestAttribute String subject){
        MessageResult<SchoolInfo> msgResult = new MessageResult<>();

        try {
            schoolInfo.setUpdateBy(subject);
            schoolInfo.setUpdateTime(new Date());

            //保存code
            String code = schoolInfo.getId();
            if("1".equals(schoolInfo.getType())){
                //为学校,保存 上级id + , + 自身ID
                code = schoolInfo.getParentId() + "," + code;
            }
            schoolInfo.setCode(code);
            int affectCount = schoolInfoService.update(schoolInfo);



            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(schoolInfo);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("数据库更新失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

	@ApiOperation(value="删除")
    @PostMapping("delete/{id}")
    public MessageResult<Integer> delete(@PathVariable("id") String id,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            SchoolInfo schoolInfo = schoolInfoService.get(id);
            schoolInfo.setDelFlag(true);
            schoolInfo.setUpdateBy(subject);
            schoolInfo.setUpdateTime(new Date());

            //如果为校区修改包含学校的code
            if("2".equals(schoolInfo.getType())){
                List<SchoolInfo> siList = schoolInfoService.findByParentId(schoolInfo.getId());
                for(SchoolInfo si : siList){
                    si.setParentId(null);
                    si.setCode(si.getId());
                    schoolInfoService.updateParentIdNull(si);
                }
            }

            int affectCount = schoolInfoService.update(schoolInfo);

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }


    @ApiOperation(value="批量删除")
    @PostMapping("batchDelete")
    public MessageResult<Integer> batchDelete(@RequestBody List<String> idList,@RequestAttribute String subject){
        MessageResult<Integer> msgResult = new MessageResult<>();

        try {
            int affectCount = 0;

            for (String id : idList) {
                SchoolInfo schoolInfo = schoolInfoService.get(id);
                schoolInfo.setDelFlag(true);
                schoolInfo.setUpdateBy(subject);
                schoolInfo.setUpdateTime(new Date());

                //如果为校区修改包含学校的code
                if("2".equals(schoolInfo.getType())){
                    List<SchoolInfo> siList = schoolInfoService.findByParentId(schoolInfo.getId());
                    for(SchoolInfo si : siList){
                        si.setParentId(null);
                        si.setCode(si.getId());
                        schoolInfoService.update(si);
                    }
                }

                affectCount += schoolInfoService.update(schoolInfo);
            }

            if (affectCount > 0) {
                msgResult.setResult(true);
                msgResult.setData(affectCount);
            } else {
                msgResult.setResult(false);
                msgResult.setMessage("删除失败");
            }
        }
        catch(Exception ex){
            logger.error(ex.getMessage(),ex);

            msgResult.setResult(false);
            msgResult.setMessage(ex.getMessage());
        }

        return msgResult;
    }

    @ApiOperation(value="列表")
    @RequestMapping(value = "pageList",method = RequestMethod.POST)
    public MessageResult<Map> pageList(
            String id,
            @RequestParam(value="name",defaultValue="") String name,
            @RequestParam(value="enrollmentType",defaultValue="") String enrollmentType,
            @RequestParam(value="type",defaultValue="") String type,
            @RequestParam(value="pageIndex",defaultValue="1") int pageIndex,
            @RequestParam(value="pageSize",defaultValue="20") int pageSize,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<Map> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("a.type_","desc"));
        sortList.add(new Sort("b.school_code","asc"));
        sortList.add(new Sort("a.school_code","asc"));

        if (StringUtils.isNotEmpty(id)) {
            searchParams.put("id","%" + id + "%");
        }

        if (StringUtils.isNotEmpty(name)) {
            searchParams.put("name" , "%" + name + "%");
        }

        if (StringUtils.isNotEmpty(enrollmentType)) {
            searchParams.put("enrollmentType" , enrollmentType);
        }

        if (StringUtils.isNotEmpty(type)) {
            searchParams.put("type" , type);
        }

        if (!userService.hasRole(subject,"SYSADMIN")) {
            User user = userService.get(subject);
            searchParams.put("code", "%" + user.getSchoolId() + "%");
        }

        Page<SchoolInfo> page = schoolInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);
        for(SchoolInfo school : page.getResult()){
            String emTypes = school.getEnrollmentType();
            if(StringUtils.isEmpty(emTypes)){
                continue;
            }
            String[] eTypes =  emTypes.split(",");
            String eTypesName = "";
            for(String eType : eTypes){
                if(StringUtils.isNotEmpty(eType)){
                    eTypesName += dataDictionaryService.findNameByCatalogNameAndValue("招生类型",eType) + ",";
                }
            }
            school.setEnrollmentTypeName(eTypesName.substring(0,eTypesName.length()-1));
        }

        msgResult.setResult(true);
        msgResult.setData(PojoUtils.pageWrapper(page));

        return msgResult;
    }

    @ApiOperation(value="校园列表")
    @RequestMapping(value = "list",method = RequestMethod.POST)
    public MessageResult<List> list(
            String id,
            String type,
            String enrollmentType,
            @RequestAttribute String subject){

        //当前用户ID
        System.out.println(subject);

        MessageResult<List> msgResult = new MessageResult<>();

        Map<String,Object> searchParams = new HashMap<>();

        List<Sort> sortList = new ArrayList<>();
        sortList.add(new Sort("parent_id","asc"));
        sortList.add(new Sort("code_","asc"));

        if (!userService.hasRole(subject,"SYSADMIN")) {
            User user = userService.get(subject);
            searchParams.put("code", "%" + user.getSchoolId() + "%");
        }
        if(StringUtils.isNotEmpty(type)){
            searchParams.put("type", type);
        }
        if(StringUtils.isNotEmpty(enrollmentType)){
            searchParams.put("enrollmentType", "%" + enrollmentType + "%");
        }

        Page<SchoolInfo> page = schoolInfoService.pageSearch(searchParams,0,10000,false,sortList);

        msgResult.setResult(true);
        msgResult.setData(page);

        return msgResult;
    }

    @ApiOperation(value="查询子单位")
    @RequestMapping(value = "children",method = RequestMethod.POST)
    public MessageResult<List<SchoolInfo>> children(String parentId){
        MessageResult<List<SchoolInfo>> msgResult = new MessageResult<>();

        try {
            List<SchoolInfo> list = schoolInfoService.findByParentId(parentId);

            List<SchoolInfo> dtoList = new ArrayList<>();

            for (SchoolInfo companyInfo : list) {
                SchoolInfo dto = new SchoolInfo();

                PojoUtils.map(companyInfo,dto);

                Boolean hasChildren = schoolInfoService.hasChildren(companyInfo.getId());
                dto.setHasChildren(hasChildren);

                dtoList.add(dto);
            }

            msgResult.setResult(true);
            msgResult.setData(dtoList);
        }
        catch (Exception ex){
            msgResult.setResult(false);
            logger.error(ex.getMessage(),ex);
        }

        return msgResult;
    }
}
