package com.jpsoft.campus.modules.common.controller;

import com.jpsoft.campus.modules.common.dto.MessageResult;
import com.jpsoft.campus.modules.common.utils.DES3;
import com.jpsoft.campus.modules.common.utils.JwtUtil;
import com.jpsoft.campus.modules.sys.entity.Role;
import com.jpsoft.campus.modules.sys.entity.User;
import com.jpsoft.campus.modules.sys.entity.UserRole;
import com.jpsoft.campus.modules.sys.service.UserRoleService;
import com.jpsoft.campus.modules.sys.service.UserService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestAttribute;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;
import javax.servlet.http.HttpSession;
import java.util.List;

@Slf4j
@RestController
public class JwtsUserController {
    @Value("${jwt.secret}")
    private String jwtSecret;

    @Autowired
    private UserService userService;

    @Autowired
    private UserRoleService userRoleService;


    @PostMapping("/login")
    @ApiOperation(value="登录获取token,在swagger ui中获取token时将写入session，调用其它接口时不用再设置header")
    @ApiImplicitParams({
            @ApiImplicitParam(name="userName", paramType="query", required=true, value="用户名"),
            @ApiImplicitParam(name="password", paramType="query", required=true, value="密码")
    })
    public MessageResult<String> login(String userName, String password, @ApiIgnore HttpSession session){
        MessageResult<String> messageResult = new MessageResult<>();

        try {
            log.warn("用户登录:" + userName);

            User user = userService.findByUserName(userName);

            DES3 des3 = new DES3();

            String passwordEnc = des3.encrypt(jwtSecret,password);

            if(user!=null && passwordEnc.equals(user.getPassword())){

                //生成token
                String token = JwtUtil.createToken(jwtSecret,user.getId(),DateTime.now().plusHours(6).toDate());

                session.setAttribute("token",token);

                messageResult.setResult(true);
                messageResult.setData(token);
            }
            else{
                messageResult.setResult(false);
                messageResult.setMessage("用户不存在或密码错误！");
            }

        }
        catch(Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @GetMapping("/userInfo")
    public MessageResult<User> userInfo(@RequestAttribute String subject){
        MessageResult<User> messageResult = new MessageResult<>();

        try {
            User user = userService.get(subject);
            //添加用户包含的角色信息
            List<String> roleList = userRoleService.findRoleNameByUserId(user.getId());
            user.setRoles(roleList);

            messageResult.setResult(true);
            messageResult.setData(user);
        }
        catch(Exception ex){
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }
}
