package com.jpsoft.campus.modules.common.controller;

import com.jpsoft.campus.config.OSSConfig;
import com.jpsoft.campus.modules.common.dto.MessageResult;
import com.jpsoft.campus.modules.common.utils.OSSUtil;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.HashMap;
import java.util.Map;

@RestController
public class fileUploadController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private OSSConfig ossConfig;

    @PostMapping("uploadPicture")
    @ApiOperation(value="照片上传")
    @ApiImplicitParams({
            @ApiImplicitParam(name="subFolder",value = "路径",required = true,paramType = "form"),
            @ApiImplicitParam(name = "photoFile",value = "员工照片", required = true,paramType="form", dataType = "__file")
    })
    public MessageResult<Map> uploadPicture(String subFolder, MultipartFile photoFile){
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            String rawFileUrl = OSSUtil.upload(ossConfig,"/" + subFolder,photoFile.getOriginalFilename(),photoFile.getInputStream());
            String signedFileUrl = OSSUtil.generatePresignedUrl(ossConfig,rawFileUrl,3600, 1024);

            Map<String,Object> dataMap = new HashMap<>();
            dataMap.put("rawFileUrl", rawFileUrl);
            dataMap.put("signedFileUrl", signedFileUrl);

            messageResult.setResult(true);
            messageResult.setData(dataMap);
            messageResult.setCode(200);
        } catch (Exception e) {
            logger.error(e.getMessage(),e);

            messageResult.setResult(false);
            messageResult.setMessage(e.getMessage());
        }

        return messageResult;
    }
}
