package com.jpsoft.campus.modules.mobile.controller;

import com.github.pagehelper.Page;
import com.jpsoft.campus.modules.base.entity.*;
import com.jpsoft.campus.modules.base.service.*;
import com.jpsoft.campus.modules.common.dto.MessageResult;
import com.jpsoft.campus.modules.common.dto.Sort;
import com.jpsoft.campus.modules.common.utils.PojoUtils;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 问卷保存
 *
 */
@Slf4j
@RestController
@RequestMapping("/mobile/collectApi")
public class CollectApiController {

    @Autowired
    private CollectService collectService;
    @Autowired
    private ApplicationPrimaryService applicationPrimaryService;
    @Autowired
    private StudentInfoService studentInfoService;
    @Autowired
    private PersonInfoService personInfoService;
    @Autowired
    private PersonStudentService personStudentService;


    @PostMapping("findCollect")
    @ApiOperation(value = "查询是否有填写问卷")
    public MessageResult<Map> findCollect(@RequestAttribute String subject){
        MessageResult<Map> messageResult = new MessageResult<>();
        try {
            PersonInfo personInfo = personInfoService.get(subject);
            if(personInfo == null){
                throw new Exception("未找到用户");
            }

            List<ApplicationPrimary> APlist = applicationPrimaryService.findByPersonId(personInfo.getId());
            List<StudentInfo> studentInfos = new ArrayList<>();
            if(APlist.size() > 0) {
                for(ApplicationPrimary ap : APlist){
                    StudentInfo studentInfo = studentInfoService.get(ap.getStudentId());
                    if(studentInfo != null){
                        studentInfos.add(studentInfo);
                    }
                }
            }

            //是否已填写
            Boolean isWirte = false;

            Collect collect = collectService.findByPersonId(personInfo.getId());
            if(collect != null){
                isWirte = true;
            }

            HashMap map = new HashMap();
            map.put("isWirte",isWirte);
            map.put("studentInfos",studentInfos);

            messageResult.setCode(200);
            messageResult.setData(map);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setCode(500);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("saveCollect")
    @ApiOperation(value = "保存问卷")
    public MessageResult<Map> saveCollect(
            String studentId,
            Boolean bit1,
            String text1,
            String text2,
            String text3,
            String text4,
            String text5,
            @RequestAttribute String subject){
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(subject);
            if(personInfo == null){
                throw new Exception("未找到用户");
            }

            List<ApplicationPrimary> APlist = applicationPrimaryService.findByPersonId(personInfo.getId());
            List<String> studentIds = new ArrayList<>();
            if(APlist.size() > 0) {
                for(ApplicationPrimary ap : APlist){
                    StudentInfo studentInfo = studentInfoService.get(ap.getStudentId());
                    if(studentInfo != null){
                        studentIds.add(studentInfo.getId());
                    }
                }
            }

            Collect collect = collectService.findByPersonId(personInfo.getId());
            if(collect == null){
                collect = new Collect();
                collect.setId(UUID.randomUUID().toString());
                collect.setCreateBy(subject);
                collect.setCreateTime(new Date());
                collect.setDelFlag(false);
                collect.setBit1(bit1);
                collect.setText1(text1);
                collect.setText2(text2);
                collect.setText3(text3);
                collect.setText4(text4);
                collect.setText5(text5);
                collect.setPersonId(subject);
                collect.setStudentId(studentId);
                collectService.insert(collect);
            }else{
                collect.setUpdateBy(subject);
                collect.setUpdateTime(new Date());
                collect.setBit1(bit1);
                collect.setText1(text1);
                collect.setText2(text2);
                collect.setText3(text3);
                collect.setText4(text4);
                collect.setText5(text5);
                collect.setPersonId(subject);
                collect.setStudentId(studentId);
                collectService.update(collect);
            }

            messageResult.setCode(200);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setCode(500);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }
}
