package com.jpsoft.campus.modules.mobile.controller;

import com.jpsoft.campus.modules.base.entity.EnrollControl;
import com.jpsoft.campus.modules.base.entity.NewsInfo;
import com.jpsoft.campus.modules.base.entity.SchoolInfo;
import com.jpsoft.campus.modules.base.service.EnrollControlService;
import com.jpsoft.campus.modules.base.service.SchoolInfoService;
import com.jpsoft.campus.modules.base.service.SchoolNumberService;
import com.jpsoft.campus.modules.common.dto.MessageResult;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;

/**
 * @author 墨鱼_mo
 * @date 2020-7-17 13:24
 */
@Slf4j
@RestController
@RequestMapping("/mobile/enrollControlApi")
public class EnrollControlApiController {

    @Autowired
    private EnrollControlService enrollControlService;

    @Autowired
    private SchoolInfoService schoolInfoService;

    @Autowired
    private SchoolNumberService schoolNumberService;


    @PostMapping("enableEnroll")
    @ApiOperation(value = "验证能否报名")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "applicationId", value = "applicationId", required = false, paramType = "query"),
            @ApiImplicitParam(name = "schoolType", value = "学校类型（1：小学，2：中学）", required = true, paramType = "query"),
            @ApiImplicitParam(name = "studentType", value = "学生类型（A,B,C,D,E）", required = true, paramType = "query"),
            @ApiImplicitParam(name = "token", value = "令牌", paramType = "query"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "query")
    })
    public MessageResult<Boolean> enableEnroll(String applicationId, String schoolType, String studentType, String token, @RequestAttribute String subject) {

        MessageResult<Boolean> messageResult = new MessageResult<>();

        boolean result = true;
        try {

            //if (StringUtils.isBlank(applicationId)) {
                List<SchoolInfo> schoolInfoList = schoolInfoService.findByEnrollmentType(schoolType);

                if (StringUtils.isNotBlank(studentType)) {
                    List<EnrollControl> enrollControls = enrollControlService.findBySchoolTypeAndStudentType(schoolType, studentType);
                    //   List<SchoolInfo> schoolList = schoolInfoService.findActiveSchoolByEnrollmentType(studentType,schoolType);


                    int i = 0;
                    if (enrollControls.size() == schoolInfoList.size()) {
                        for (EnrollControl enrollControl : enrollControls) {
                            if (enrollControl.getEndTime().compareTo(new Date()) < 0) {
                                  i++;
                            }
                        }

                        if (i == schoolInfoList.size()) {
                            throw new Exception(studentType + "类学生报名通道已全部关闭");
                        }


                    }

                }/*else {

                    List<String> studentTypeList = new ArrayList<>();
                    studentTypeList.add("A");
                    studentTypeList.add("B");
                    studentTypeList.add("C");
                    if ("2".equals(schoolType)){
                        studentTypeList.add("D");
                        studentTypeList.add("E");
                    }

                    for (String type : studentTypeList){
                        List<EnrollControl> enrollControls = enrollControlService.findBySchoolTypeAndStudentType(schoolType, studentType);
                    }




                }*/
            //}


            messageResult.setCode(200);
            messageResult.setData(result);
            messageResult.setResult(true);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);

            messageResult.setData(false);
            messageResult.setCode(500);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("schoolTypeEnableEnroll")
    @ApiOperation(value = "验证小学中学能否报名")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "schoolType", value = "学校类型（1：小学，2：中学）", required = true, paramType = "query"),
            @ApiImplicitParam(name = "token", value = "令牌", paramType = "query"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "query")
    })
    public MessageResult<Boolean> schoolTypeEnableEnroll(String schoolType, String token, @RequestAttribute String subject) {

        MessageResult<Boolean> messageResult = new MessageResult<>();

        boolean result = true;
        try {


                List<SchoolInfo> schoolInfoList = schoolInfoService.findByEnrollmentType(schoolType);

            List<String> studentTypeList = new ArrayList<>();
            studentTypeList.add("A");
            studentTypeList.add("B");
            studentTypeList.add("C");
            if ("2".equals(schoolType)){
                studentTypeList.add("D");
                studentTypeList.add("E");
            }

            int i = 0;
            for (String type : studentTypeList){
                List<EnrollControl> enrollControls = enrollControlService.findBySchoolTypeAndStudentType(schoolType, type);

                if (enrollControls.size() == schoolInfoList.size()) {
                    for (EnrollControl enrollControl : enrollControls) {
                        if (enrollControl.getStartTime().compareTo(new Date()) > 0) {
                            i++;
                        }else if(enrollControl.getEndTime().compareTo(new Date()) < 0){
                            i++;
                        }
                    }

                }else {
                    break;
                }
            }
            if (i == schoolInfoList.size()*studentTypeList.size()){
                result = false;
            }


            messageResult.setCode(200);
            messageResult.setData(result);
            messageResult.setResult(true);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);

            messageResult.setData(false);
            messageResult.setCode(500);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("transferEnable")
    @ApiOperation(value = "验证能否插班转学报名")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "schoolType", value = "学校类型（1：小学，2：中学）", required = true, paramType = "query"),
            @ApiImplicitParam(name = "token", value = "令牌", paramType = "query"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "query")
    })
    public MessageResult<Boolean> transferEnable(String schoolType, String token, @RequestAttribute String subject) {

        MessageResult<Boolean> messageResult = new MessageResult<>();

        boolean result = true;
        try {


            List<SchoolInfo> schoolInfoList = schoolInfoService.findByEnrollmentType(schoolType);

            List<String> studentTypeList = new ArrayList<>();
            studentTypeList.add("Y");
            studentTypeList.add("Z");

            int i = 0;
            for (String type : studentTypeList){
                List<EnrollControl> enrollControls = enrollControlService.findBySchoolTypeAndStudentType(schoolType, type);

                if (enrollControls.size() == schoolInfoList.size()) {
                    for (EnrollControl enrollControl : enrollControls) {
                        if (enrollControl.getStartTime().compareTo(new Date()) > 0) {
                            i++;
                        }else if(enrollControl.getEndTime().compareTo(new Date()) < 0){
                            i++;
                        }
                    }

                }else {
                    break;
                }
            }
            if (i == schoolInfoList.size()*studentTypeList.size()){
                result = false;
            }


            messageResult.setCode(200);
            messageResult.setData(result);
            messageResult.setResult(true);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);

            messageResult.setData(false);
            messageResult.setCode(500);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }
}
