package com.jpsoft.campus.modules.mobile.controller;

import cn.hutool.core.date.DateUtil;
import com.jpsoft.campus.modules.base.dao.ApplicationMiddleDAO;
import com.jpsoft.campus.modules.base.dto.ApplicationMiddleDTO;
import com.jpsoft.campus.modules.base.dto.ApplicationPrimaryDTO;
import com.jpsoft.campus.modules.base.dto.AttachmentDTO;
import com.jpsoft.campus.modules.base.dto.SchoolInfoDTO;
import com.jpsoft.campus.modules.base.entity.*;
import com.jpsoft.campus.modules.base.service.*;
import com.jpsoft.campus.modules.common.dto.MessageResult;
import com.jpsoft.campus.modules.common.dto.Sort;
import com.jpsoft.campus.modules.common.utils.CheckIdCard;
import com.jpsoft.campus.modules.sys.entity.DataDictionary;
import com.jpsoft.campus.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.parsing.BeanEntry;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author 墨鱼_mo
 * @date 2020-7-1 13:34
 */
@Slf4j
@RestController
@RequestMapping("/mobile/middleSchoolApi")
public class MiddleSchoolApiController {

    @Autowired
    private DataDictionaryService dataDictionaryService;

    @Autowired
    private ApplicationMiddleService applicationMiddleService;

    @Autowired
    private SchoolRelationService schoolRelationService;

    @Autowired
    private SchoolInfoService schoolInfoService;

    @Autowired
    private ApplicationMiddleDAO applicationMiddleDAO;

    @Autowired
    private StudentFilesService studentFilesService;

    @Autowired
    private StudentInfoService studentInfoService;

    @Autowired
    private ApprovalInfoService approvalInfoService;

    @Autowired
    private PersonStudentService personStudentService;

    @Autowired
    private ApplicationPrimaryService applicationPrimaryService;

    @Autowired
    private EnrollControlService enrollControlService;


    @GetMapping("queryRelationshipList")
    @ApiOperation(value = "查询学生关系列表")
    public MessageResult<List> queryRelationshipList() {
        MessageResult<List> messageResult = new MessageResult<>();

        try {
            List<DataDictionary> ddList = dataDictionaryService.findByCatalogName("学生关系");

            messageResult.setData(ddList);
            messageResult.setResult(true);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @GetMapping("queryMiddleListByPrimary")
    @ApiOperation(value = "按照小学id查询对口中学")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "primaryId", value = "小学id", required = true, paramType = "query"),
            @ApiImplicitParam(name = "type", value = "学生类型", required = true, paramType = "query"),
            @ApiImplicitParam(name = "token", value = "令牌", paramType = "query"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "query")
    })
    public MessageResult<List> queryMiddleListByPrimary(String primaryId,String type, String token, @RequestAttribute String subject) {
        MessageResult<List> messageResult = new MessageResult<>();

        try {
            if (StringUtils.isBlank(primaryId)) {
                throw new Exception("小学id为空");
            }

            List<SchoolRelation> list = schoolRelationService.findByPrimaryId(primaryId);
            List<SchoolInfoDTO> dataList = new ArrayList<>();
            if (list.size() > 0) {
                for (SchoolRelation schoolRelation : list) {
                    SchoolInfo schoolInfo = schoolInfoService.get(schoolRelation.getMiddleId());
                    if (schoolInfo != null) {
                        Map<String,Object> searchParams = new HashMap<>();
                        searchParams.put("schoolId",schoolInfo.getId());
                        searchParams.put("schoolType","2");
                        searchParams.put("studentType",type);
                        List<Sort> sortList = new ArrayList<>();
                        sortList.add(new Sort("create_time","asc"));
                        List<EnrollControl> enrollControls = enrollControlService.searchParam(searchParams,sortList);
                        SchoolInfoDTO schoolInfoDTO = new SchoolInfoDTO();
                        if (enrollControls.size()>0){
                            if (enrollControls.get(0).getEndTime().compareTo(new Date()) >0){
                                BeanUtils.copyProperties(schoolInfo, schoolInfoDTO);
                                schoolInfoDTO.setRemark(schoolRelation.getRemark());
                                dataList.add(schoolInfoDTO);
                            }
                        }else {
                            BeanUtils.copyProperties(schoolInfo, schoolInfoDTO);
                            schoolInfoDTO.setRemark(schoolRelation.getRemark());
                            dataList.add(schoolInfoDTO);
                        }



                    }


                }
            }


            messageResult.setData(dataList);
            messageResult.setResult(true);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @GetMapping("querySchoolList")
    @ApiOperation(value = "查询学校列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "报名类型", required = true, paramType = "query")
    })
    public MessageResult<Map> querySchoolList(String type,String token, @RequestAttribute String subject){
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            List<SchoolInfo> schoolList = schoolInfoService.findActiveSchoolByEnrollmentType(type,"2");

            HashMap map = new HashMap();
            map.put("list",schoolList);
            messageResult.setData(map);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("validateIdCard")
    @ApiOperation(value = "验证身份证")
    public MessageResult<String> validateIdCard(String applicationId, String idCard, @RequestAttribute String subject) {
        MessageResult<String> messageResult = new MessageResult<>();

        try {
            CheckIdCard cic = new CheckIdCard(idCard);

            if (!cic.validate()) {
                throw new Exception("身份证无效!");
            }


            //todo 根据身份证查询当前学生是否已报名
            boolean exist = applicationMiddleService.existByIdCard(idCard, applicationId);

            if (exist) {
                throw new Exception("当前身份证号已报名!请在【报名查询】菜单中查看报名信息。");
            }

            StudentInfo studentInfo = studentInfoService.findByIdCard(idCard);
            if (studentInfo != null){
                List<ApplicationPrimary> applicationPrimaryList = applicationPrimaryService.findByStudentId(studentInfo.getId());
                if (applicationPrimaryList.size()>0){
                    for (ApplicationPrimary applicationPrimary : applicationPrimaryList){
                        if (applicationPrimary.getStatus().equals("10") || applicationPrimary.getStatus().equals("20") || applicationPrimary.getStatus().equals("30")){
                            throw new Exception("有待处理的小学报名记录，不可报名中学");
                        }
                    }
                }

            }

            messageResult.setResult(true);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("validateIdCardYZ")
    @ApiOperation(value = "验证身份证")
    public MessageResult<String> validateIdCardYZ(String applicationId, String idCard, @RequestAttribute String subject) {
        MessageResult<String> messageResult = new MessageResult<>();

        try {
            CheckIdCard cic = new CheckIdCard(idCard);

            if (!cic.validate()) {
                throw new Exception("身份证无效!");
            }


            //todo 根据身份证查询当前学生是否已报名
            boolean exist = applicationMiddleService.existByIdCard(idCard, applicationId);

            if (exist) {
                throw new Exception("当前身份证号已报名!请在【报名查询】菜单中查看报名信息。");
            }

            //todo 根据身份证查询当前学生是否已报名
            boolean exist2 = applicationPrimaryService.existByIdCard(idCard,applicationId);
            if (exist2){
                throw new Exception("当前身份证号已报名!");
            }

            StudentInfo studentInfo = studentInfoService.findByIdCard(idCard);
            if (studentInfo != null){
                List<ApplicationPrimary> applicationPrimaryList = applicationPrimaryService.findByStudentId(studentInfo.getId());
                if (applicationPrimaryList.size()>0){
                    for (ApplicationPrimary applicationPrimary : applicationPrimaryList){
                        if(!applicationPrimary.getId().equals(applicationId)) {
                            if (applicationPrimary.getStatus().equals("10") || applicationPrimary.getStatus().equals("20") || applicationPrimary.getStatus().equals("30")) {
                                throw new Exception("有待处理的小学报名记录，不可报名中学");
                            }
                        }
                    }
                }

                List<ApplicationMiddle> applicationMiddleList = applicationMiddleService.findByStudentId(studentInfo.getId());
                if (applicationMiddleList.size()>0){
                    for (ApplicationMiddle applicationMiddle : applicationMiddleList){
                        if(!applicationMiddle.getId().equals(applicationId)) {
                            if (applicationMiddle.getStatus().equals("10") || applicationMiddle.getStatus().equals("20") || applicationMiddle.getStatus().equals("30")) {
                                throw new Exception("有待处理的中学报名记录，不可报名小学");
                            }
                        }
                    }
                }
            }

            messageResult.setResult(true);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @GetMapping("queryRegistrationFileList")
    @ApiOperation(value = "查询报名资料相关附件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "报名类型", paramType = "query"),
            @ApiImplicitParam(name = "applicationId", value = "申请表编号", paramType = "query"),
            @ApiImplicitParam(name = "token", value = "令牌", paramType = "query"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "query")
    })
    public MessageResult<List> queryRegistrationFileList(String type, String applicationId, String token, @RequestAttribute String subject) {
        MessageResult<List> messageResult = new MessageResult<>();

        try {
            List<DataDictionary> ddList = dataDictionaryService.findByCatalogName("其他相关证明（" + type + "）");

            List<Map> mapList = new ArrayList<>();

            for (DataDictionary dd : ddList) {
                Map<String, Object> map = new HashMap<>();
                map.put("id", dd.getId());
                map.put("name", dd.getName());
                map.put("type", dd.getExtended1()); //0-非必填、1-该分类或该附件必填、2-只需要上传一类附件
                map.put("remark", dd.getExtended2()); //备注
                map.put("description", dd.getExtended3());

                //findByCatalogName
                List<DataDictionary> attachmentList = dataDictionaryService.findByParentId(dd.getId());

                List<AttachmentDTO> attachmentDTOList = new ArrayList<>();

                for (DataDictionary attachment : attachmentList) {
                    AttachmentDTO attachmentDTO = new AttachmentDTO();

                    attachmentDTO.setId(attachment.getId());
                    attachmentDTO.setName(attachment.getName());
                    attachmentDTO.setType(attachment.getExtended1());
                    attachmentDTO.setRemark(attachment.getExtended2());

                    List<String> files = new ArrayList<>();

                    if (StringUtils.isNotEmpty(applicationId)) {
                        //如果申请表号已存在则查询已上传附件
                        List<StudentFiles> fileList = studentFilesService.findByApplicationIdAndFileType(applicationId, attachment.getId());

                        if (fileList != null && fileList.size() > 0) {
                            files = fileList.stream().map(StudentFiles::getFileUrl).collect(Collectors.toList());
                        }
                    }

                    attachmentDTO.setFiles(files);

                    attachmentDTOList.add(attachmentDTO);
                }

                map.put("attachments", attachmentDTOList);

                mapList.add(map);
            }

            messageResult.setData(mapList);
            messageResult.setResult(true);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @GetMapping("queryStudentCertificationList")
    @ApiOperation(value = "查询学生证明及附件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "报名类型", paramType = "query"),
            @ApiImplicitParam(name = "applicationId", value = "申请表编号", paramType = "query"),
            @ApiImplicitParam(name = "token", value = "令牌", paramType = "query"),
            @ApiImplicitParam(name = "subject", value = "目标(不传)", paramType = "query")
    })
    public MessageResult<List> queryStudentCertificationList(String type, String applicationId, String token, @RequestAttribute String subject) {
        MessageResult<List> messageResult = new MessageResult<>();

        try {
            List<Map> mapList = new ArrayList<>();
            Map<String, Object> dataMap = new HashMap<>();

            DataDictionary studentCertDict = dataDictionaryService.findByName("学生相关证明（" + type + "）");

            if (studentCertDict != null) {


                dataMap.put("id", studentCertDict.getId());
                dataMap.put("name", studentCertDict.getName());
                dataMap.put("type", studentCertDict.getExtended1()); //0-非必填、1-该分类或该附件必填、2-只需要上传一类附件
                dataMap.put("remark", studentCertDict.getExtended2()); //备注
                dataMap.put("description", studentCertDict.getExtended3());

                List<DataDictionary> attachments = dataDictionaryService.findByParentId(studentCertDict.getId());

                List<AttachmentDTO> attachmentDTOList = new ArrayList<>();

                for (DataDictionary attachment : attachments) {
                    AttachmentDTO attachmentDTO = new AttachmentDTO();

                    attachmentDTO.setId(attachment.getId());
                    attachmentDTO.setName(attachment.getName());
                    attachmentDTO.setType(attachment.getExtended1());
                    attachmentDTO.setRemark(attachment.getExtended2());

                    if("Y".equals(type)) {
                        attachmentDTO.setNumber(attachment.getExtended3());
                        if(StringUtils.isNotEmpty(attachment.getExtended2())) {
                            attachmentDTO.setNotes(Arrays.asList(attachment.getExtended2().split(",")));
                        }
                    }else if("Z".equals(type)){
                        attachmentDTO.setNumber(attachment.getExtended3());
                        if(StringUtils.isNotEmpty(attachment.getExtended2())) {
                            attachmentDTO.setNotes(Arrays.asList(attachment.getExtended2().split(",")));
                        }
                    }

                    List<String> files = new ArrayList<>();

                    if (StringUtils.isNotEmpty(applicationId)) {
                        //如果申请表号已存在则查询已上传附件
                        List<StudentFiles> fileList = studentFilesService.findByApplicationIdAndFileType(applicationId, attachment.getId());

                        if (fileList != null && fileList.size() > 0) {
                            files = fileList.stream().map(StudentFiles::getFileUrl).collect(Collectors.toList());
                        }
                    }

                    attachmentDTO.setFiles(files);

                    attachmentDTOList.add(attachmentDTO);
                }

                dataMap.put("attachments", attachmentDTOList);

            }

            mapList.add(dataMap);

            messageResult.setData(mapList);
            messageResult.setResult(true);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("submit")
    @ApiOperation(value = "提交报名信息")
    public MessageResult<Map> submit(@RequestBody ApplicationMiddleDTO applicationMiddleDTO,
                                        @RequestAttribute String subject){
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            //todo 将报名信息保存到各张表中
            ApplicationMiddle applicationMiddle = null;
            StudentInfo studentInfo = null;

            //根据身份证查询当前学生是否已报名
            boolean exist = applicationMiddleService.existByIdCard(applicationMiddleDTO.getIdCard(),applicationMiddleDTO.getApplicationId());

            if (exist){
                throw new Exception("当前身份证号已报名!");
            }

            if (StringUtils.isNotBlank(applicationMiddleDTO.getApplicationId())){
                applicationMiddle = applicationMiddleService.get(applicationMiddleDTO.getApplicationId());
            }


            //前端未传applicationMiddleDTO.studentId,改用idCard
            if (StringUtils.isNotEmpty(applicationMiddleDTO.getIdCard())) {
                studentInfo = studentInfoService.findByIdCard(applicationMiddleDTO.getIdCard());
            }

            if (studentInfo==null) {
                studentInfo = new StudentInfo();
                studentInfo.setId(UUID.randomUUID().toString());
                studentInfo.setName(applicationMiddleDTO.getName());
                studentInfo.setOldName(applicationMiddleDTO.getOldName());
                studentInfo.setCardType(applicationMiddleDTO.getCardType());
                studentInfo.setIdCard(applicationMiddleDTO.getIdCard());
                studentInfo.setSex(applicationMiddleDTO.getSex());
                studentInfo.setBirthday(applicationMiddleDTO.getBirthday());
                studentInfo.setNation(applicationMiddleDTO.getNation());
                studentInfo.setAddress(applicationMiddleDTO.getAddress());
                studentInfo.setDocumentUrl(applicationMiddleDTO.getDocumentUrl());
                studentInfo.setType(applicationMiddleDTO.getStudentType());
                studentInfo.setStudentNumber(applicationMiddleDTO.getStudentNumber());
                studentInfo.setOldClass(applicationMiddleDTO.getOldClass());

                studentInfo.setCreateTime(new Date());
                studentInfo.setCreateBy(subject);
                studentInfo.setDelFlag(false);

                studentInfoService.insert(studentInfo);
            }
            else{
                studentInfo.setName(applicationMiddleDTO.getName());
                studentInfo.setOldName(applicationMiddleDTO.getOldName());
                studentInfo.setCardType(applicationMiddleDTO.getCardType());
                studentInfo.setIdCard(applicationMiddleDTO.getIdCard());
                studentInfo.setSex(applicationMiddleDTO.getSex());
                studentInfo.setBirthday(applicationMiddleDTO.getBirthday());
                studentInfo.setNation(applicationMiddleDTO.getNation());
                studentInfo.setAddress(applicationMiddleDTO.getAddress());
                studentInfo.setDocumentUrl(applicationMiddleDTO.getDocumentUrl());
                studentInfo.setType(applicationMiddleDTO.getStudentType());
                studentInfo.setStudentNumber(applicationMiddleDTO.getStudentNumber());
                studentInfo.setOldClass(applicationMiddleDTO.getOldClass());


                studentInfo.setUpdateTime(new Date());
                studentInfo.setUpdateBy(subject);

                studentInfoService.update(studentInfo);
            }


            if (applicationMiddle == null) {
                if("Y".equals(applicationMiddleDTO.getType()) || "Z".equals(applicationMiddleDTO.getType())){
                    //转校
                    applicationMiddle = new ApplicationMiddle();

                    applicationMiddle.setId(UUID.randomUUID().toString());
                    applicationMiddle.setType(applicationMiddleDTO.getType());
                    applicationMiddle.setCategoryId(applicationMiddleDTO.getCategoryId());
                    applicationMiddle.setChooseReason(applicationMiddleDTO.getChooseReason());
                    applicationMiddle.setPersonId(subject);
                    applicationMiddle.setStudentId(studentInfo.getId());

                    applicationMiddle.setStreetId(applicationMiddleDTO.getStreetId());
                    applicationMiddle.setCommunityId(applicationMiddleDTO.getCommunityId());
                    applicationMiddle.setSchoolId(applicationMiddleDTO.getSchoolId());
                    applicationMiddle.setSchoolNumberId(applicationMiddleDTO.getSchoolNumberId());

                    applicationMiddle.setCreateTime(new Date());
                    applicationMiddle.setCreateBy(subject);
                    applicationMiddle.setDelFlag(false);
                    applicationMiddle.setStatus("10"); //待招生办审批

                    applicationMiddleService.insert(applicationMiddle);

                    //添加审批记录
                    ApprovalInfo approvalInfo = new ApprovalInfo();
                    approvalInfo.setId(UUID.randomUUID().toString());
                    approvalInfo.setApplicationId(applicationMiddle.getId());
                    approvalInfo.setOperation("提交转校报名申请");
                    approvalInfo.setCreateBy(subject);
                    approvalInfo.setCreateTime(new Date());

                    approvalInfoService.insert(approvalInfo);
                } else {
                    applicationMiddle = new ApplicationMiddle();

                    applicationMiddle.setId(UUID.randomUUID().toString());
                    applicationMiddle.setType(applicationMiddleDTO.getType());
                    applicationMiddle.setCategoryId(applicationMiddleDTO.getCategoryId());
                    applicationMiddle.setChooseReason(applicationMiddleDTO.getChooseReason());
                    applicationMiddle.setPersonId(subject);
                    applicationMiddle.setStudentId(studentInfo.getId());

                    applicationMiddle.setStreetId(applicationMiddleDTO.getStreetId());
                    applicationMiddle.setCommunityId(applicationMiddleDTO.getCommunityId());
                    applicationMiddle.setSchoolId(applicationMiddleDTO.getSchoolId());

                    applicationMiddle.setCreateTime(new Date());
                    applicationMiddle.setCreateBy(subject);
                    applicationMiddle.setDelFlag(false);
                    applicationMiddle.setStatus("20"); //待招生办审批

                    applicationMiddleService.insert(applicationMiddle);

                    //添加审批记录
                    ApprovalInfo approvalInfo = new ApprovalInfo();
                    approvalInfo.setId(UUID.randomUUID().toString());
                    approvalInfo.setApplicationId(applicationMiddle.getId());
                    approvalInfo.setOperation("提交入学报名申请");
                    approvalInfo.setCreateBy(subject);
                    approvalInfo.setCreateTime(new Date());

                    approvalInfoService.insert(approvalInfo);
                }
            }else {
                if("Y".equals(applicationMiddleDTO.getType()) || "Z".equals(applicationMiddleDTO.getType())){
                    applicationMiddle.setType(applicationMiddleDTO.getType());
                    applicationMiddle.setCategoryId(applicationMiddleDTO.getCategoryId());
                    applicationMiddle.setPersonId(subject);
                    applicationMiddle.setStudentId(studentInfo.getId());

                    applicationMiddle.setStreetId(applicationMiddleDTO.getStreetId());
                    applicationMiddle.setChooseReason(applicationMiddleDTO.getChooseReason());
                    applicationMiddle.setCommunityId(applicationMiddleDTO.getCommunityId());
                    applicationMiddle.setSchoolId(applicationMiddleDTO.getSchoolId());
                    applicationMiddle.setSchoolNumberId(applicationMiddleDTO.getSchoolNumberId());
                    applicationMiddle.setStatus("10"); //退回后编辑-》待网上初审
                    applicationMiddle.setUpdateBy(subject);
                    applicationMiddle.setUpdateTime(new Date());

                    applicationMiddleService.update(applicationMiddle);

                    //添加审批记录
                    ApprovalInfo approvalInfo = new ApprovalInfo();
                    approvalInfo.setId(UUID.randomUUID().toString());
                    approvalInfo.setApplicationId(applicationMiddle.getId());
                    approvalInfo.setOperation("编辑后再次提交转校报名申请");
                    approvalInfo.setCreateBy(subject);
                    approvalInfo.setCreateTime(new Date());

                    approvalInfoService.insert(approvalInfo);
                } else {
                    applicationMiddle.setType(applicationMiddleDTO.getType());
                    applicationMiddle.setCategoryId(applicationMiddleDTO.getCategoryId());
                    applicationMiddle.setPersonId(subject);
                    applicationMiddle.setStudentId(studentInfo.getId());

                    applicationMiddle.setStreetId(applicationMiddleDTO.getStreetId());
                    applicationMiddle.setChooseReason(applicationMiddleDTO.getChooseReason());
                    applicationMiddle.setCommunityId(applicationMiddleDTO.getCommunityId());
                    applicationMiddle.setSchoolId(applicationMiddleDTO.getSchoolId());
                    applicationMiddle.setStatus("20"); //退回后编辑-》待网上初审
                    applicationMiddle.setUpdateBy(subject);
                    applicationMiddle.setUpdateTime(new Date());

                    applicationMiddleService.update(applicationMiddle);

                    //添加审批记录
                    ApprovalInfo approvalInfo = new ApprovalInfo();
                    approvalInfo.setId(UUID.randomUUID().toString());
                    approvalInfo.setApplicationId(applicationMiddle.getId());
                    approvalInfo.setOperation("编辑后再次提交入学报名申请");
                    approvalInfo.setCreateBy(subject);
                    approvalInfo.setCreateTime(new Date());

                    approvalInfoService.insert(approvalInfo);
                }
            }

            PersonStudent personStudent = personStudentService.findByPersonIdAndStudentId(subject,studentInfo.getId());

            if (personStudent==null){
                personStudent = new PersonStudent();

                personStudent.setId(UUID.randomUUID().toString());
                personStudent.setPersonId(subject);
                personStudent.setStudentId(studentInfo.getId());
                personStudent.setRelationship(applicationMiddleDTO.getRelationship());

                personStudent.setCreateTime(new Date());
                personStudent.setCreateBy(subject);
                personStudent.setDelFlag(false);

                personStudentService.insert(personStudent);
            }
            else{
                personStudent.setPersonId(subject);
                personStudent.setStudentId(studentInfo.getId());
                personStudent.setRelationship(applicationMiddleDTO.getRelationship());

                personStudent.setUpdateBy(subject);
                personStudent.setUpdateTime(new Date());

                personStudentService.update(personStudent);
            }

            //报名相关附件
            if (applicationMiddleDTO.getRegistrationFileList()!=null){
                saveAttachmentList(applicationMiddle.getId(),studentInfo.getId(),applicationMiddleDTO.getRegistrationFileList(),subject);
            }

            //保存学生相关证明
            if (applicationMiddleDTO.getStudentCertificationFileList()!=null){
                saveAttachmentList(applicationMiddle.getId(),studentInfo.getId(),applicationMiddleDTO.getStudentCertificationFileList(),subject);
            }

//            Date startTime = DateUtil.beginOfYear(new Date());
//            Date endTime = new Date();

            Map<String,Object> searchParams = new HashMap<>();
            searchParams.put("schoolId",applicationMiddle.getSchoolId());
            searchParams.put("delFlag",false);
//            searchParams.put("startCreateTime",startTime);
//            searchParams.put("endCreateTime",endTime);
            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time","desc"));

            List<ApplicationMiddle> applicationMiddleList = applicationMiddleDAO.search(searchParams,sortList);

            Map<String,Object> map = new HashMap<>();
            map.put("id",applicationMiddle.getId());
            map.put("num",applicationMiddleList.size());

            messageResult.setData(map);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    private void saveAttachmentList(String applicationId,String studentId, List<AttachmentDTO> attachmentList, String subject){
        for (AttachmentDTO attachmentDTO : attachmentList) {
            studentFilesService.deleteByApplicationIdAndFileType(applicationId, attachmentDTO.getId());

            for (int i=0;i<attachmentDTO.getFiles().size();i++) {
                String fileUrl = attachmentDTO.getFiles().get(i);

                StudentFiles studentFiles = new StudentFiles();

                studentFiles.setId(UUID.randomUUID().toString());
                studentFiles.setApplicationId(applicationId);
                studentFiles.setStudentId(studentId);
                studentFiles.setFileType(attachmentDTO.getId());
                studentFiles.setFileUrl(fileUrl);
                studentFiles.setDelFlag(false);
                studentFiles.setSortNo(i+1);

                studentFiles.setCreateTime(new Date());
                studentFiles.setCreateBy(subject);

                studentFilesService.insert(studentFiles);
            }
        }
    }


}
