package com.jpsoft.campus.modules.mobile.controller;

import com.github.pagehelper.Page;
import com.jpsoft.campus.modules.base.dto.NewsDTO;
import com.jpsoft.campus.modules.base.dto.SchoolInfoDTO;
import com.jpsoft.campus.modules.base.entity.NewsInfo;
import com.jpsoft.campus.modules.base.entity.SchoolInfo;
import com.jpsoft.campus.modules.base.service.NewsInfoService;
import com.jpsoft.campus.modules.common.dto.MessageResult;
import com.jpsoft.campus.modules.common.dto.Sort;
import com.jpsoft.campus.modules.common.utils.PojoUtils;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author 墨鱼_mo
 * @date 2020-6-23 14:59
 */
@Slf4j
@RestController
@RequestMapping("/mobile/newsApi")
public class NewsApiController {

    @Autowired
    private NewsInfoService newsInfoService;



    @PostMapping("rotationNewsList")
    @ApiOperation(value = "手机端新闻(轮播新闻)")
    public MessageResult<Map> rotationNewsList(){

        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            List<NewsInfo> newsInfoList = newsInfoService.findByTypeAndStatus("1","1");


            HashMap map = new HashMap();
            map.put("list",newsInfoList);
            messageResult.setCode(200);
            messageResult.setData(map);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setCode(500);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("normalNewsList")
    @ApiOperation(value = "手机端新闻(非轮播新闻)")
    public MessageResult<Map> normalNewsList(
            @RequestParam(value = "type", defaultValue = "1") String type,
            @RequestParam(value = "pageIndex", defaultValue = "1") int pageIndex,
            @RequestParam(value = "pageSize", defaultValue = "10") int pageSize){

        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            Map<String,Object> searchParams = new HashMap<>();
            searchParams.put("type",type);
            searchParams.put("status","1");

            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("sort_no","asc"));
            sortList.add(new Sort("create_time","desc"));

            Page<NewsInfo> page = newsInfoService.pageSearch(searchParams,pageIndex,pageSize,true,sortList);


            messageResult.setCode(200);
            messageResult.setData(PojoUtils.pageWrapper(page));
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setCode(500);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("detail")
    @ApiOperation(value = "新闻详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id", required = true, paramType = "form")
    })
    public MessageResult<NewsInfo> detail(String id){

        MessageResult<NewsInfo> messageResult = new MessageResult<>();

        try {
            NewsInfo newsInfo = newsInfoService.get(id);
            if (newsInfo == null){
                throw new Exception("新闻不存在");
            }


            messageResult.setCode(200);
            messageResult.setData(newsInfo);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setCode(500);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


}
