package com.jpsoft.campus.modules.mobile.controller;

import com.jpsoft.campus.modules.base.entity.*;
import com.jpsoft.campus.modules.base.service.ApplicationMiddleService;
import com.jpsoft.campus.modules.base.service.ApplicationPrimaryService;
import com.jpsoft.campus.modules.base.service.StudentInfoService;
import net.sf.json.JSONObject;
import com.jpsoft.campus.config.BaiduConfig;
import com.jpsoft.campus.config.OSSConfig;
import com.jpsoft.campus.modules.base.service.PersonInfoService;
import com.jpsoft.campus.modules.common.dto.MessageResult;
import com.jpsoft.campus.modules.common.utils.*;
import com.jpsoft.campus.modules.sys.entity.SysLog;
import com.jpsoft.campus.modules.sys.service.SysLogService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.web.bind.annotation.*;
import sun.misc.BASE64Decoder;

import java.io.ByteArrayInputStream;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;

@Slf4j
@RestController
@RequestMapping("/mobile/personInfoApi")
public class PersonInfoApiController {
    @Value("${jwt.secret}")
    private String jwtSecret;

    @Autowired
    private BaiduConfig baiduConfig;

    @Autowired
    private OSSConfig ossConfig;

    @Autowired
    private ValueOperations<String, Object> valueOperations;

    @Autowired
    private SysLogService sysLogService;

    @Autowired
    private PersonInfoService personInfoService;

    @Autowired
    private StudentInfoService studentInfoService;

    @Autowired
    private ApplicationPrimaryService applicationPrimaryService;

    @Autowired
    private ApplicationMiddleService applicationMiddleService;

    @PostMapping("getVerifyCode")
    @ApiOperation(value = "获取短信验证码(公开接口)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "phone", value = "电话", required = true, paramType = "form")
    })
    public MessageResult<String> getVerifyCode(String phone) {
        MessageResult<String> messageResult = new MessageResult<>();

        SysLog sysLog = new SysLog();
        sysLog.setPointcut("获取短信验证码");
        sysLog.setUserId(phone);
        sysLog.setCreateTime(new Date());

        try {
            String key = "SMS_" + phone;

            String verifyCode = (String) valueOperations.get(key);

            if (StringUtils.isEmpty(verifyCode)) {
                verifyCode = SMSUtil.generateNumberString(6);
                JSONObject verifyCodeJSON = new JSONObject();
                verifyCodeJSON.put("code", verifyCode);

                sysLog.setData(verifyCode);

                messageResult = SMSUtil.send(phone, "荆州开发区网上入学", "SMS_49390047", verifyCodeJSON);

                //设置当前用户的验证码,1分钟内有效
                valueOperations.set(key, verifyCode, 1, TimeUnit.MINUTES);
            }
            else{
                throw new Exception("1分钟内不能重复发送!");
            }

            messageResult.setResult(true);
            messageResult.setCode(200);

            sysLog.setRemark("发送结果:" + messageResult.getMessage());
        } catch (Exception ex) {
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());

            sysLog.setRemark(ex.getMessage());
        }

        sysLogService.insert(sysLog);

        return messageResult;
    }


    @PostMapping("login")
    @ApiOperation(value = "登录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "phone", value = "电话号码", required = true, paramType = "form"),
            @ApiImplicitParam(name = "verifyCode", value = "验证码", required = true, paramType = "form"),
            @ApiImplicitParam(name = "openId", value = "openId", required = true, paramType = "form")
    })
    public MessageResult<Map> login(String phone, String verifyCode,String openId) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.findByPhone(phone);

            if (personInfo==null){
                throw new Exception("用户未注册!");
            }

            String smsKey = "SMS_" + phone;

            String beforeVerifyCode = (String) valueOperations.get(smsKey);

            if (StringUtils.isEmpty(beforeVerifyCode)) {
                throw new Exception("验证码已过期!");
            }

            if (!beforeVerifyCode.equals(verifyCode)) {
                throw new Exception("验证码错误!");
            }

            personInfo.setOpenId(openId);
            personInfo.setUpdateBy(personInfo.getId());
            personInfo.setUpdateTime(new Date());

            personInfoService.update(personInfo);

            String token = JwtUtil.createToken(jwtSecret, String.valueOf(personInfo.getId()), DateTime.now().plusHours(6).toDate());

            Map<String,Object> dataMap = new HashMap<>();

            dataMap.put("person",personInfo);
            dataMap.put("token", token);

            messageResult.setData(dataMap);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("register")
    @ApiOperation(value = "注册")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "name", value = "姓名", required = true, paramType = "form"),
            @ApiImplicitParam(name = "idCard", value = "身份证号码", required = false, paramType = "form"),
            @ApiImplicitParam(name = "phone", value = "电话号码", required = false, paramType = "form"),
            @ApiImplicitParam(name = "verifyCode", value = "验证码", required = true, paramType = "form"),
            @ApiImplicitParam(name = "openId", value = "微信openId", paramType = "form"),
    })
    public MessageResult<Map> register(String name, String idCard, String phone, String verifyCode, String openId) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            CheckIdCard cic = new CheckIdCard(idCard);

            if (!cic.validate()) {
                throw new Exception("身份证验证失败!");
            }
            if (StringUtils.isBlank(openId)){
                throw new Exception("openId获取失败，请刷新后操作");
            }

            PersonInfo personInfo = personInfoService.findByPhone(phone);

            if (personInfo!=null){
                throw new Exception("当前手机号已注册!");
            }

            personInfo = personInfoService.findByIdCard(idCard);

            if (personInfo!=null){
                throw new Exception("当前身份证号已注册!");
            }

            String smsKey = "SMS_" + phone;

//            String beforeVerifyCode = (String) valueOperations.get(smsKey);
//
//            if (StringUtils.isEmpty(beforeVerifyCode)) {
//                throw new Exception("验证码已过期!");
//            }
//
//            if (!beforeVerifyCode.equals(verifyCode)) {
//                throw new Exception("验证码错误!");
//            }

            personInfo = new PersonInfo();
            personInfo.setId(UUID.randomUUID().toString());
            personInfo.setName(name);
            personInfo.setIdCard(idCard.trim().toUpperCase());
            personInfo.setPhone(phone);
            personInfo.setOpenId(openId);

            personInfo.setDelFlag(false);
            personInfo.setCreateTime(new Date());

            personInfoService.insert(personInfo);

            String token = JwtUtil.createToken(jwtSecret, personInfo.getId(), DateTime.now().plusHours(6).toDate());

            Map<String,Object> dataMap = new HashMap<>();

            dataMap.put("person",personInfo);
            dataMap.put("token", token);

            messageResult.setData(dataMap);
            messageResult.setResult(true);
            messageResult.setCode(200);
        } catch (Exception ex) {
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("findByOpenId")
    @ApiOperation(value = "通过openId查询人员(公开接口)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "openId", value = "微信openId", required = true, paramType = "form")
    })
    public MessageResult<Map> findByOpenId(String openId) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.findByOpenId(openId);

            if(personInfo!=null) {
                Map<String, Object> dataMap = new HashMap<>();

                dataMap.put("person", personInfo);
                String token = JwtUtil.createToken(jwtSecret, String.valueOf(personInfo.getId()), DateTime.now().plusHours(6).toDate());
                dataMap.put("token", token);

                messageResult.setData(dataMap);
                messageResult.setResult(true);
                messageResult.setCode(200);
            }
            else{
                messageResult.setResult(true);
                messageResult.setMessage("人员未注册");
                messageResult.setCode(404);
            }
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("getUserInfo")
    @ApiOperation(value = "获取UIAS用户信息")
    public MessageResult<Map> getUserInfo(@RequestAttribute String subject) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            PersonInfo personInfo = personInfoService.get(subject);
            if(personInfo!=null) {
                Map<String, Object> dataMap = new HashMap<>();
                String name = personInfo.getName();;

                char[] r = name.toCharArray();
                String realname1 = null;
                if(r.length ==1){
                    realname1 = name;
                }

                if(r.length == 2){
                    realname1 = name.replaceFirst(name.substring(1),"*");
                }

                if (r.length > 2) {
                    realname1 = name.replaceFirst(name.substring(1,r.length-1) ,"*");
                }

                personInfo.setName(realname1);

                dataMap.put("person", personInfo);
//                String token = JwtUtil.createToken(jwtSecret, String.valueOf(personInfo.getId()), DateTime.now().plusHours(6).toDate());
//                dataMap.put("token", token);
                messageResult.setData(dataMap);
                messageResult.setResult(true);
            }
            else{
                messageResult.setResult(true);
                messageResult.setMessage("未登录");
            }
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("documentImageOCR")
    @ResponseBody
    @ApiOperation(value = "证件图片识别")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "documentType", value = "证件类型(1:身份证，2:户口本)", required = true, paramType = "form"),
            @ApiImplicitParam(name = "base64Data", value = "员工照片base64编码", required = true, paramType = "form")
    })
    public MessageResult<Map> documentImageOCR(String documentType, String base64Data) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            String baiduATKey = "baidu_accessToken";
            String accessToken = (String) valueOperations.get(baiduATKey);

            if (StringUtils.isEmpty(accessToken)) {
                JSONObject jsonObject = BaiduOCR.getAuth(baiduConfig.getAk(), baiduConfig.getSk());

                log.warn(jsonObject.toString());

                accessToken = jsonObject.getString("access_token");
                long expiredSeconds = jsonObject.getLong("expires_in");

                valueOperations.set(baiduATKey, accessToken, expiredSeconds, TimeUnit.SECONDS);
            }

            String[] arr = base64Data.split(",");
            String imageBase64Data = arr[1];

            Map<String,Object> dataMap = new HashMap<>();

            if ("1".equals(documentType)) {
                JSONObject jsonObject = BaiduOCR.idcard(accessToken, imageBase64Data);

                if(jsonObject!=null) {
                    log.warn(jsonObject.toString());

                    if(!jsonObject.containsKey("words_result")){
                        throw new Exception("身份证未识别!");
                    }
                }
                else{
                    throw new Exception("身份证未识别!");
                }

                JSONObject wordsResult = jsonObject.getJSONObject("words_result");

                try{
                    dataMap.put("address", wordsResult.getJSONObject("住址").getString("words"));
                    dataMap.put("idCard", wordsResult.getJSONObject("公民身份号码").getString("words"));
                    dataMap.put("birthday",wordsResult.getJSONObject("出生").getString("words"));
                    dataMap.put("name",wordsResult.getJSONObject("姓名").getString("words"));
                    dataMap.put("sex",wordsResult.getJSONObject("性别").getString("words"));
                    dataMap.put("nation",wordsResult.getJSONObject("民族").getString("words"));
                }catch (Exception e){
                    throw new Exception("身份证识别错误");
                }


            } else {
                JSONObject jsonObject = BaiduOCR.householdRegister(accessToken, imageBase64Data);

                if(jsonObject!=null) {
                    log.warn(jsonObject.toString());

                    if(!jsonObject.containsKey("words_result")){
                        throw new Exception("户口本未识别!");
                    }
                }
                else{
                    throw new Exception("户口本未识别!");
                }

                JSONObject wordsResult = jsonObject.getJSONObject("words_result");

                try{
                    dataMap.put("address", wordsResult.getJSONObject("BirthAddress").getString("words"));
                    dataMap.put("idCard", wordsResult.getJSONObject("CardNo").getString("words"));
                    dataMap.put("birthday",wordsResult.getJSONObject("Birthday").getString("words"));
                    dataMap.put("name",wordsResult.getJSONObject("Name").getString("words"));
                    dataMap.put("sex",wordsResult.getJSONObject("Sex").getString("words"));
                    dataMap.put("nation",wordsResult.getJSONObject("Nation").getString("words"));
                }catch (Exception e){
                    throw new Exception("户口本识别错误");
                }

            }

            if (dataMap.get("birthday")!=null){
                SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");

                try {
                    sdf.parse(dataMap.get("birthday").toString());
                }
                catch (Exception ex){
                    dataMap.put("birthday", "");
                    log.error(ex.getMessage());
                }
            }

            BASE64Decoder decoder = new BASE64Decoder();
            byte[] imgData = decoder.decodeBuffer(imageBase64Data);

            for (int i = 0; i < imgData.length; ++i) {
                if (imgData[i] < 0) {// 调整异常数据
                    imgData[i] += 256;
                }
            }

            ByteArrayInputStream inputStream = new ByteArrayInputStream(imgData);

            String documentUrl = OSSUtil.upload(ossConfig, "/person", "1.jpg", inputStream,"仅用于开发区网上入学报名使用");

            dataMap.put("documentUrl", documentUrl);

            messageResult.setData(dataMap);
            messageResult.setResult(true);
        } catch (Exception e) {
            log.error(e.getMessage(), e);

            messageResult.setResult(false);
            messageResult.setMessage(e.getMessage());
        }

        return messageResult;
    }

    @PostMapping("uploadBase64")
    @ResponseBody
    @ApiOperation(value = "照片上传")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "photoName", value = "照片名称", required = true, paramType = "form"),
            @ApiImplicitParam(name = "photoBase64Data", value = "员工照片base64编码", required = true, paramType = "form")
    })
    public MessageResult<Map> uploadBase64(String photoName, String photoBase64Data) {
        MessageResult<Map> messageResult = new MessageResult<>();

        SysLog sysLog = new SysLog();
        sysLog.setPointcut("人员照片上传");

        try {
            sysLog.setRemark("base64 照片大小：" + photoBase64Data.length());

            //前50个字符
            if (StringUtils.isNotEmpty(photoBase64Data)) {
                sysLog.setData(photoBase64Data.substring(0, Math.min(photoBase64Data.length(), 50)));
            }

            BASE64Decoder decoder = new BASE64Decoder();

            String[] arr = photoBase64Data.split(",");

            byte[] imgData = decoder.decodeBuffer(arr[1]);

            for (int i = 0; i < imgData.length; ++i) {
                if (imgData[i] < 0) {// 调整异常数据
                    imgData[i] += 256;
                }
            }

            ByteArrayInputStream inputStream = new ByteArrayInputStream(imgData);

            if (StringUtils.isEmpty(photoName)) {
                photoName = "1.jpg";
            }

            //图片加水印"仅用于开发区网上入学报名使用"

            String rawFileUrl = OSSUtil.upload(ossConfig, "/person", photoName, inputStream,"仅用于开发区网上入学报名使用");
            String signedFileUrl = OSSUtil.generatePresignedUrl(ossConfig, rawFileUrl, 3600, 1024);

            Map<String, Object> dataMap = new HashMap<>();
            dataMap.put("rawFileUrl", rawFileUrl);
            dataMap.put("signedFileUrl", signedFileUrl);

            messageResult.setResult(true);
            messageResult.setData(dataMap);
            messageResult.setCode(200);

            sysLog.setUrl(rawFileUrl);
        } catch (Exception e) {
            log.error(e.getMessage(), e);

            messageResult.setResult(false);
            messageResult.setMessage(e.getMessage());

            sysLog.setRemark(sysLog.getRemark() + "," + e.getMessage());
        }

        sysLog.setCreateTime(new Date());
        sysLogService.insert(sysLog);

        return messageResult;
    }

    @GetMapping("queryStudentList")
    @ApiOperation(value = "查询学生列表")
    public MessageResult<List> queryStudentList(@RequestAttribute String subject){
        MessageResult<List> messageResult = new MessageResult<>();

        //List<StudentInfo> studentList = studentInfoService.findStudentByPersonId(subject);

        try {
            //先查小学
            List<ApplicationPrimary> applicationPrimaries = applicationPrimaryService.findByPersonId(subject);

            List<ApplicationMiddle> applicationMiddles = applicationMiddleService.findByPersonId(subject);

            List<Map> mapList = new ArrayList<>();

            if (applicationPrimaries.size()>0){
                for (ApplicationPrimary applicationPrimary : applicationPrimaries) {
                    Map<String, Object> map = new HashMap<>();

                    map.put("applicationId", applicationPrimary.getId());

                    String studentId = applicationPrimary.getStudentId();

                    StudentInfo studentInfo = studentInfoService.get(studentId);

                    map.put("name", studentInfo.getName());

                    mapList.add(map);
                }
            }

            if (applicationMiddles.size()>0){
                for (ApplicationMiddle applicationMiddle : applicationMiddles) {
                    Map<String, Object> map = new HashMap<>();

                    map.put("applicationId", applicationMiddle.getId());

                    String studentId = applicationMiddle.getStudentId();

                    StudentInfo studentInfo = studentInfoService.get(studentId);

                    map.put("name", studentInfo.getName());

                    mapList.add(map);
                }
            }


            messageResult.setData(mapList);
            messageResult.setResult(true);
        }
        catch(Exception ex){
            log.error(ex.getMessage(), ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("uploadUser")
    public MessageResult<Map> uploadUser(UserInformationCity userInformationCity) {
        MessageResult<Map> messageResult = new MessageResult<>();

        try {

            if(userInformationCity != null){
                messageResult.setResult(true);
            }else{
                messageResult.setResult(false);
            }

        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("dwonUserMessage")
    @ApiOperation(value = "查询市级学生数据")
    public MessageResult<List<UserInformationCity>> dwonUserMessage() {
        MessageResult<List<UserInformationCity>> messageResult = new MessageResult<>();

        try {
            List<UserInformationCity> primaryList = applicationPrimaryService.findOffice();
            messageResult.setData(primaryList);
            messageResult.setResult(true);

        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("updateUploadStatus")
    @ApiOperation(value = "修改市级学生上传状态")
    public MessageResult<Boolean> updateUploadStatus(String id,String code) {
        MessageResult<Boolean> messageResult = new MessageResult<>();
        try {
            ApplicationPrimary applicationPrimary = applicationPrimaryService.findByOfficeCode(id);
            applicationPrimary.setUploadStatus(code);
            applicationPrimaryService.update(applicationPrimary);
            messageResult.setResult(true);

        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    /**
    @PostMapping("dwonProvincialUserMessage")
    @ApiOperation(value = "查询省级学生数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "applyNo", value = "applyNo", required = true, paramType = "query"),
            @ApiImplicitParam(name = "limit", value = "查询数量", paramType = "query"),
    })
    public MessageResult<List<Map>> dwonProvincialUserMessage(
            @RequestParam(value="applyNo",defaultValue="") String applyNo,
            @RequestParam(value="limit",defaultValue="100") Integer limit) {
        MessageResult<List<Map>> messageResult = new MessageResult<>();

        try {
            if(StringUtils.isEmpty(applyNo)){
                throw new Exception("未传applyNo");
            }
            List<UserInformationProvincial> primaryList = applicationPrimaryService.findOfficeByProvincial(limit);
            List<Map> returnList = new ArrayList<>();
            for(UserInformationProvincial provincial : primaryList){


                UserInformationProvincialSubDeclares subDeclares = new UserInformationProvincialSubDeclares();
                subDeclares.setApplicantName(provincial.getApplicantName());
                subDeclares.setApplicantPhone(provincial.getApplicantPhone());
                subDeclares.setApplyNo(applyNo);
                subDeclares.setLiceseNo(provincial.getLicenseNo());
                subDeclares.setTargetId(provincial.getTargetId());
                subDeclares.setTargetName(provincial.getTargetName());

                UserInformationProvincialFileDeclares fileDeclares = new UserInformationProvincialFileDeclares();
                fileDeclares.setApplyNo(applyNo);

                List<UserInformationProvincialFileDeclares> fileDeclaresList = new ArrayList<>();
                fileDeclaresList.add(fileDeclares);

//                subDeclares.setFileDeclares(fileDeclaresList);

                List<UserInformationProvincialSubDeclares> subDeclaresList = new ArrayList<>();
                subDeclaresList.add(subDeclares);
//                provincial.setSubDeclares(subDeclaresList);


                Map userMap = new HashMap();
                userMap.put("declare",provincial);
                returnList.add(userMap);
            }

            messageResult.setData(returnList);
            messageResult.setResult(true);


        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @PostMapping("updateProvincialStatus")
    @ApiOperation(value = "省级修改学生上传状态")
    public MessageResult<Boolean> updateProvincialStatus(String id,String code,String applyNo,String subNos) {
        MessageResult<Boolean> messageResult = new MessageResult<>();
        try {
            ApplicationPrimary applicationPrimary = applicationPrimaryService.get(id);
            applicationPrimary.setCode(code);
            applicationPrimary.setApplyNo(applyNo);
            applicationPrimary.setSubNos(subNos);
            applicationPrimaryService.update(applicationPrimary);
            messageResult.setResult(true);

        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    **/
}
