package com.jpsoft.campus.modules.mobile.controller;

import cn.hutool.core.date.DateUtil;
import com.jpsoft.campus.modules.base.dao.ApplicationPrimaryDAO;
import com.jpsoft.campus.modules.base.dto.ApplicationPrimaryDTO;
import com.jpsoft.campus.modules.base.dto.AttachmentDTO;
import com.jpsoft.campus.modules.base.entity.*;
import com.jpsoft.campus.modules.base.service.*;
import com.jpsoft.campus.modules.common.dto.MessageResult;
import com.jpsoft.campus.modules.common.dto.Sort;
import com.jpsoft.campus.modules.common.utils.CheckIdCard;
import com.jpsoft.campus.modules.sys.entity.DataDictionary;
import com.jpsoft.campus.modules.sys.service.DataDictionaryService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@RestController
@RequestMapping("/mobile/primarySchoolApi")
public class PrimarySchoolApiController {
    @Autowired
    private DataDictionaryService dataDictionaryService;

    @Autowired
    private DistrictInfoService districtInfoService;

    @Autowired
    private ApplicationPrimaryDAO applicationPrimaryDAO;

    @Autowired
    private StudentInfoService studentInfoService;

    @Autowired
    private StudentFilesService studentFilesService;

    @Autowired
    private SchoolInfoService schoolInfoService;

    @Autowired
    private ApplicationPrimaryService applicationPrimaryService;

    @Autowired
    private PersonStudentService personStudentService;

    @Autowired
    private ApprovalInfoService approvalInfoService;

    @Autowired
    private ApplicationMiddleService applicationMiddleService;




    @GetMapping("queryStreetList")
    @ApiOperation(value = "查询街道列表")
    public MessageResult<List> queryStreetList(){
        MessageResult<List> messageResult = new MessageResult<>();

        try {
            List<DataDictionary> ddList = dataDictionaryService.findByCatalogName("街道");

            messageResult.setData(ddList);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @GetMapping("queryCommunityList")
    @ApiOperation(value = "查询社区列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "报名类型", required = true, paramType = "query"),
            @ApiImplicitParam(name = "streetId", value = "街道编号", required = true, paramType = "query")
    })
    public MessageResult<List> queryCommunityList(String type,String streetId){
        MessageResult<List> messageResult = new MessageResult<>();

        try {
            List<DistrictInfo> list = districtInfoService.findActiveByStreetId("1",type,streetId);

//            for (DistrictInfo districtInfo : list) {
//                if (StringUtils.isEmpty(districtInfo.getSchoolName())){
//                    districtInfo.setSchoolId2(districtInfo.getSchoolId());
//                    //说明学校已报名截止
//                    districtInfo.setSchoolId(null);
//
//                }
//            }

            messageResult.setData(list);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @GetMapping("queryDistrictSchoolList")
    @ApiOperation(value = "查询社区学校列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "社区编号", required = false, paramType = "query"),
            @ApiImplicitParam(name = "schoolType", value = "学校类型 (1小学2初中,不传默认为小学)", required = false, paramType = "query")
    })
    public MessageResult<List> queryDistrictSchoolList(String id,
                                                       @RequestParam(value="schoolType",defaultValue="1") String schoolType,
                                                       String token){
        MessageResult<List> messageResult = new MessageResult<>();

        try {
            //DistrictInfo districtInfo = districtInfoService.get(id);
            //List<String> schoolIdList = Arrays.asList(districtInfo.getSchoolId().split(","));
//            List<SchoolInfo> list = new ArrayList<>();
//            if (schoolIdList.size()>0){
//                for (String schoolId : schoolIdList){
//                    SchoolInfo schoolInfo = schoolInfoService.get(schoolId);
//                    if (schoolInfo != null){
//                        if(schoolInfo.getEnrollmentType().contains(schoolType) && schoolInfo.getIsOpen()){
//                            list.add(schoolInfo);
//                        }
//                    }
//                }
//            }
            List<SchoolInfo> schoolIdList = schoolInfoService.findByEnrollmentType(schoolType);

            messageResult.setData(schoolIdList);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }



    @GetMapping("querySchoolList")
    @ApiOperation(value = "查询学校列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "报名类型", required = true, paramType = "query")
    })
    public MessageResult<List> querySchoolList(String type){
        MessageResult<List> messageResult = new MessageResult<>();

        try {
            List<SchoolInfo> schoolList = schoolInfoService.findActiveSchoolByEnrollmentType(type,"1");
            messageResult.setData(schoolList);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @GetMapping("queryRelationshipList")
    @ApiOperation(value = "查询学生关系列表")
    public MessageResult<List> queryRelationshipList(){
        MessageResult<List> messageResult = new MessageResult<>();

        try {
            List<DataDictionary> ddList = dataDictionaryService.findByCatalogName("学生关系");

            messageResult.setData(ddList);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @GetMapping("queryRegistrationFileList")
    @ApiOperation(value = "查询报名资料相关附件")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "type", value = "报名类型", paramType = "query"),
        @ApiImplicitParam(name = "applicationId", value = "申请表编号", paramType = "query")
    })
    public MessageResult<List> queryRegistrationFileList(String type,String applicationId){
        MessageResult<List> messageResult = new MessageResult<>();

        try {
            List<DataDictionary> ddList = dataDictionaryService.findByCatalogName(type + "类生证明");

            List<Map> mapList = new ArrayList<>();

            for (DataDictionary dd : ddList) {
                Map<String,Object> map = new HashMap<>();
                map.put("id", dd.getId());
                map.put("name" ,dd.getName());
                map.put("type" , dd.getExtended1()); //0-非必填、1-该分类或该附件必填、2-只需要上传一类附件
                map.put("remark", dd.getExtended2()); //备注
                map.put("description" , dd.getExtended3());

                //findByCatalogName
                List<DataDictionary> attachmentList = dataDictionaryService.findByParentId(dd.getId());

                List<AttachmentDTO> attachmentDTOList = new ArrayList<>();

                for (DataDictionary attachment : attachmentList) {
                    AttachmentDTO attachmentDTO = new AttachmentDTO();

                    attachmentDTO.setId(attachment.getId());
                    attachmentDTO.setName(attachment.getName());
                    attachmentDTO.setType(attachment.getExtended1());
                    attachmentDTO.setRemark(attachment.getExtended2());

                    List<String> files = new ArrayList<>();

                    if(StringUtils.isNotEmpty(applicationId)){
                        //如果申请表号已存在则查询已上传附件
                        List<StudentFiles> fileList = studentFilesService.findByApplicationIdAndFileType(applicationId,attachment.getId());

                        if (fileList!=null && fileList.size()>0){
                            files = fileList.stream().map(StudentFiles::getFileUrl).collect(Collectors.toList());
                        }
                    }

                    attachmentDTO.setFiles(files);

                    attachmentDTOList.add(attachmentDTO);
                }

                map.put("attachments", attachmentDTOList);

                mapList.add(map);
            }

            messageResult.setData(mapList);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @GetMapping("queryStudentCertificationList")
    @ApiOperation(value = "查询学生证明及附件")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "报名类型", paramType = "query"),
            @ApiImplicitParam(name = "applicationId", value = "申请表编号", paramType = "query")
    })
    public MessageResult<List> queryStudentCertificationList(String type,String applicationId){
        MessageResult<List> messageResult = new MessageResult<>();

        try {
            List<Map> mapList = new ArrayList<>();
            Map<String,Object> dataMap = new HashMap<>();
            DataDictionary studentCertDict = null;
            List<DataDictionary> attachments = null;
            if(StringUtils.isNotEmpty(type)) {
                if("Y".equals(type)) {
                    studentCertDict = dataDictionaryService.get("cbc52a2f-5c25-4504-8442-d6c64ea61552");
                    attachments = dataDictionaryService.findByParentId("cbc52a2f-5c25-4504-8442-d6c64ea61552");
                }else if("Z".equals(type)){
                    studentCertDict = dataDictionaryService.get("904fb0bd-e697-42bf-bea4-bea87f5b8126");
                    attachments = dataDictionaryService.findByParentId("904fb0bd-e697-42bf-bea4-bea87f5b8126");
                }else{
                    studentCertDict = dataDictionaryService.get(DataDictionary.STUDENT_RELATED_CERT_ID);
                    attachments = dataDictionaryService.findByParentId(DataDictionary.STUDENT_RELATED_CERT_ID);
                }
            }else{
                studentCertDict = dataDictionaryService.get(DataDictionary.STUDENT_RELATED_CERT_ID);
                attachments = dataDictionaryService.findByParentId(DataDictionary.STUDENT_RELATED_CERT_ID);
            }

            dataMap.put("id", studentCertDict.getId());
            dataMap.put("name", studentCertDict.getName());
            dataMap.put("type" , studentCertDict.getExtended1()); //0-非必填、1-该分类或该附件必填、2-只需要上传一类附件
            dataMap.put("remark", studentCertDict.getExtended2()); //备注
            dataMap.put("description" , studentCertDict.getExtended3());

            List<AttachmentDTO> attachmentDTOList = new ArrayList<>();

            for (DataDictionary attachment : attachments) {
                AttachmentDTO attachmentDTO = new AttachmentDTO();

                attachmentDTO.setId(attachment.getId());
                attachmentDTO.setName(attachment.getName());
                attachmentDTO.setType(attachment.getExtended1());
                attachmentDTO.setRemark(attachment.getExtended2());

                if("Y".equals(type)) {
                    attachmentDTO.setNumber(attachment.getExtended3());
                    if(StringUtils.isNotEmpty(attachment.getExtended2())) {
                        attachmentDTO.setNotes(Arrays.asList(attachment.getExtended2().split(",")));
                    }
                }else if("Z".equals(type)){
                    attachmentDTO.setNumber(attachment.getExtended3());
                    if(StringUtils.isNotEmpty(attachment.getExtended2())) {
                        attachmentDTO.setNotes(Arrays.asList(attachment.getExtended2().split(",")));
                    }
                }

                List<String> files = new ArrayList<>();

                if(StringUtils.isNotEmpty(applicationId)){
                    //如果申请表号已存在则查询已上传附件
                    List<StudentFiles> fileList = studentFilesService.findByApplicationIdAndFileType(applicationId,attachment.getId());

                    if (fileList!=null && fileList.size()>0){
                        files = fileList.stream().map(StudentFiles::getFileUrl).collect(Collectors.toList());
                    }
                }

                attachmentDTO.setFiles(files);

                attachmentDTOList.add(attachmentDTO);
            }

            dataMap.put("attachments", attachmentDTOList);

            mapList.add(dataMap);

            messageResult.setData(mapList);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("validateIdCard")
    @ApiOperation(value = "验证身份证")
    public MessageResult<String> validateIdCard(String applicationId,String idCard,
                                                String birthday,
                                                @RequestAttribute String subject){
        MessageResult<String> messageResult = new MessageResult<>();

        try {
            CheckIdCard cic = new CheckIdCard(idCard);

            if (!cic.validate()){
                throw new Exception("身份证无效!");
            }

//            DateTime birthDate = new DateTime(cic.getBirthDate());
//
//            if (!birthDate.toString("yyyy-MM-dd").equals(birthday)) {
//                throw new Exception("身份证号中出生年月与实际填写出生年月不符!");
//            }

            if (StringUtils.isNotEmpty(birthday)){
                //todo 增加检查学生是否满6岁
                DateTime startTime = DateTime.parse(birthday, DateTimeFormat.forPattern("yyyy-MM-dd"));
                DateTime endTime = DateTime.parse(DateTime.now().getYear() + "-09-01", DateTimeFormat.forPattern("yyyy-MM-dd"));

                if (startTime.compareTo(endTime.minusYears(6))>=0){
                    throw new Exception("学生未满6岁不能报名!");
                }
            }

            //todo 根据身份证查询当前学生是否已报名
            boolean exist = applicationPrimaryService.existByIdCard(idCard,applicationId);
            if (exist){
                throw new Exception("当前身份证号已报名!");
            }

            StudentInfo studentInfo = studentInfoService.findByIdCard(idCard);
            if (studentInfo != null){
                List<ApplicationMiddle> applicationMiddleList = applicationMiddleService.findByStudentId(studentInfo.getId());
                if (applicationMiddleList.size()>0){
                    for (ApplicationMiddle applicationMiddle : applicationMiddleList){
                        if (applicationMiddle.getStatus().equals("10") || applicationMiddle.getStatus().equals("20") || applicationMiddle.getStatus().equals("30")){
                            throw new Exception("有待处理的中学报名记录，不可报名小学");
                        }
                    }
                }

            }

            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("submit")
    @ApiOperation(value = "提交报名信息")
    public MessageResult<Map> submit(@RequestBody @Validated ApplicationPrimaryDTO applicationPrimaryDTO,
                                        @RequestAttribute String subject){
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            //todo 将报名信息保存到各张表中
            ApplicationPrimary applicationPrimary = null;
            StudentInfo studentInfo = null;

            //根据身份证查询当前学生是否已报名
            boolean exist = applicationPrimaryService.existByIdCard(applicationPrimaryDTO.getIdCard(),applicationPrimaryDTO.getApplicationId());

            if (exist){
                throw new Exception("当前身份证号已报名!");
            }

            applicationPrimary = applicationPrimaryService.get(applicationPrimaryDTO.getApplicationId());

            //前端未传applicationPrimaryDTO.studentId,改用idCard
            if (StringUtils.isNotEmpty(applicationPrimaryDTO.getIdCard())) {
                studentInfo = studentInfoService.findByIdCard(applicationPrimaryDTO.getIdCard());
            }

            if (studentInfo==null) {
                studentInfo = new StudentInfo();
                studentInfo.setId(UUID.randomUUID().toString());
                studentInfo.setName(applicationPrimaryDTO.getName());
                studentInfo.setOldName(applicationPrimaryDTO.getOldName());
                studentInfo.setCardType(applicationPrimaryDTO.getCardType());
                studentInfo.setIdCard(applicationPrimaryDTO.getIdCard());
                studentInfo.setSex(applicationPrimaryDTO.getSex());
                studentInfo.setBirthday(applicationPrimaryDTO.getBirthday());
                studentInfo.setNation(applicationPrimaryDTO.getNation());
                studentInfo.setAddress(applicationPrimaryDTO.getAddress());
                studentInfo.setDocumentUrl(applicationPrimaryDTO.getDocumentUrl());
                studentInfo.setType(applicationPrimaryDTO.getStudentType());
                studentInfo.setOldClass(applicationPrimaryDTO.getOldClass());
                studentInfo.setResidenceAddress(applicationPrimaryDTO.getResidenceAddress());

                studentInfo.setCreateTime(new Date());
                studentInfo.setCreateBy(subject);
                studentInfo.setDelFlag(false);

                studentInfoService.insert(studentInfo);
            }
            else{
                studentInfo.setName(applicationPrimaryDTO.getName());
                studentInfo.setOldName(applicationPrimaryDTO.getOldName());
                studentInfo.setCardType(applicationPrimaryDTO.getCardType());
                studentInfo.setIdCard(applicationPrimaryDTO.getIdCard());
                studentInfo.setSex(applicationPrimaryDTO.getSex());
                studentInfo.setBirthday(applicationPrimaryDTO.getBirthday());
                studentInfo.setNation(applicationPrimaryDTO.getNation());
                studentInfo.setAddress(applicationPrimaryDTO.getAddress());
                studentInfo.setDocumentUrl(applicationPrimaryDTO.getDocumentUrl());
                studentInfo.setType(applicationPrimaryDTO.getStudentType());
                studentInfo.setOldClass(applicationPrimaryDTO.getOldClass());
                studentInfo.setResidenceAddress(applicationPrimaryDTO.getResidenceAddress());

                studentInfo.setUpdateTime(new Date());
                studentInfo.setUpdateBy(subject);

                studentInfoService.update(studentInfo);
            }

            if (applicationPrimary==null){
                applicationPrimary = new ApplicationPrimary();

                applicationPrimary.setId(UUID.randomUUID().toString());
                applicationPrimary.setType(applicationPrimaryDTO.getType());
                applicationPrimary.setCategoryId(applicationPrimaryDTO.getCategoryId());
                applicationPrimary.setPersonId(subject);
                applicationPrimary.setStudentId(studentInfo.getId());

                applicationPrimary.setStreetId(applicationPrimaryDTO.getStreetId());
                applicationPrimary.setCommunityId(applicationPrimaryDTO.getCommunityId());
                applicationPrimary.setSchoolId(applicationPrimaryDTO.getSchoolId());
                applicationPrimary.setSchoolNumberId(applicationPrimaryDTO.getSchoolNumberId());
                applicationPrimary.setComeFrom(applicationPrimaryDTO.getComeFrom());

                applicationPrimary.setCreateTime(new Date());
                applicationPrimary.setCreateBy(subject);
                applicationPrimary.setDelFlag(false);

//                if ("B".equals(applicationPrimaryDTO.getType())){
//                    applicationPrimary.setStatus("40");
//                }else {
//                    applicationPrimary.setStatus("10"); //待网上初审
//                }
                applicationPrimary.setStatus("10"); //待网上初审


                applicationPrimaryService.insert(applicationPrimary);

                //添加审批记录
                ApprovalInfo approvalInfo = new ApprovalInfo();
                approvalInfo.setId(UUID.randomUUID().toString());
                approvalInfo.setApplicationId(applicationPrimary.getId());
                //转校
                if("Y".equals(applicationPrimaryDTO.getType()) || "Z".equals(applicationPrimaryDTO.getType())){
                    approvalInfo.setOperation("提交转校报名申请");
                }else{
                    approvalInfo.setOperation("提交入学报名申请");
                }
                approvalInfo.setCreateBy(subject);
                approvalInfo.setCreateTime(new Date());
                approvalInfo.setDelFlag(false);

                approvalInfoService.insert(approvalInfo);
            }
            else{
                applicationPrimary.setType(applicationPrimaryDTO.getType());
                applicationPrimary.setCategoryId(applicationPrimaryDTO.getCategoryId());
                applicationPrimary.setPersonId(subject);
                applicationPrimary.setStudentId(studentInfo.getId());

                applicationPrimary.setStreetId(applicationPrimaryDTO.getStreetId());
                applicationPrimary.setCommunityId(applicationPrimaryDTO.getCommunityId());
                applicationPrimary.setSchoolId(applicationPrimaryDTO.getSchoolId());
                applicationPrimary.setSchoolNumberId(applicationPrimaryDTO.getSchoolNumberId());


//                if ("B".equals(applicationPrimaryDTO.getType())){
//                    applicationPrimary.setStatus("40");
//                }else {
//                    applicationPrimary.setStatus("10"); //待网上初审
//                }
                applicationPrimary.setStatus("10"); //待网上初审
                applicationPrimary.setUpdateBy(subject);
                applicationPrimary.setUpdateTime(new Date());

                applicationPrimaryService.update(applicationPrimary);

                //添加审批记录
                ApprovalInfo approvalInfo = new ApprovalInfo();
                approvalInfo.setId(UUID.randomUUID().toString());
                approvalInfo.setApplicationId(applicationPrimary.getId());
                //转校
                if("Y".equals(applicationPrimaryDTO.getType()) || "Z".equals(applicationPrimaryDTO.getType())){
                    approvalInfo.setOperation("编辑后再次提交转校报名申请");
                }else {
                    approvalInfo.setOperation("编辑后再次提交入学报名申请");
                }
                approvalInfo.setCreateBy(subject);
                approvalInfo.setCreateTime(new Date());
                approvalInfo.setDelFlag(false);

                approvalInfoService.insert(approvalInfo);
            }

            PersonStudent personStudent = personStudentService.findByPersonIdAndStudentId(subject,studentInfo.getId());

            if (personStudent==null){
                personStudent = new PersonStudent();

                personStudent.setId(UUID.randomUUID().toString());
                personStudent.setPersonId(subject);
                personStudent.setStudentId(studentInfo.getId());
                personStudent.setRelationship(applicationPrimaryDTO.getRelationship());

                personStudent.setCreateTime(new Date());
                personStudent.setCreateBy(subject);
                personStudent.setDelFlag(false);

                personStudentService.insert(personStudent);
            }
            else{
                personStudent.setPersonId(subject);
                personStudent.setStudentId(studentInfo.getId());
                personStudent.setRelationship(applicationPrimaryDTO.getRelationship());

                personStudent.setUpdateBy(subject);
                personStudent.setUpdateTime(new Date());

                personStudentService.update(personStudent);
            }

            //报名相关附件
            if (applicationPrimaryDTO.getRegistrationFileList()!=null){
                saveAttachmentList(applicationPrimary.getId(),studentInfo.getId(),applicationPrimaryDTO.getRegistrationFileList(),subject);
            }

            //保存学生相关证明
            if (applicationPrimaryDTO.getStudentCertificationFileList()!=null){
                saveAttachmentList(applicationPrimary.getId(),studentInfo.getId(),applicationPrimaryDTO.getStudentCertificationFileList(),subject);
            }

            Date startTime = DateUtil.beginOfYear(new Date());
            Date endTime = new Date();

            Map<String,Object> searchParams = new HashMap<>();
            searchParams.put("schoolId",applicationPrimary.getSchoolId());
            searchParams.put("startCreateTime",startTime);
            searchParams.put("endCreateTime",endTime);
            List<Sort> sortList = new ArrayList<>();
            sortList.add(new Sort("create_time","desc"));

            //List<ApplicationPrimary> applicationPrimaryList = applicationPrimaryDAO.search(searchParams,sortList);

            Map<String,Object> map = new HashMap<>();
            map.put("id",applicationPrimary.getId());
            //map.put("num",applicationPrimaryList.size());

            messageResult.setData(map);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("cancel")
    @ApiOperation(value = "取消报名")
    public MessageResult<String> cancel(String applicationId, @RequestAttribute String subject){
        MessageResult<String> messageResult = new MessageResult<>();

        try {
            //todo


            if (StringUtils.isBlank(applicationId)){
                throw new Exception("几率id为空");
            }
            log.warn(applicationId);
            ApplicationPrimary applicationPrimary = applicationPrimaryService.get(applicationId);



            if (applicationPrimary == null){
                ApplicationMiddle applicationMiddle = applicationMiddleService.get(applicationId);
                applicationMiddle.setDelFlag(true);
                applicationMiddle.setUpdateBy(subject);
                applicationMiddle.setUpdateTime(new Date());

                applicationMiddleService.update(applicationMiddle);


            }else {
                applicationPrimary.setDelFlag(true);
                applicationPrimary.setUpdateBy(subject);
                applicationPrimary.setUpdateTime(new Date());

                applicationPrimaryService.update(applicationPrimary);
            }



            //添加审批记录
            ApprovalInfo approvalInfo = new ApprovalInfo();
            approvalInfo.setId(UUID.randomUUID().toString());
            approvalInfo.setApplicationId(applicationId);
            approvalInfo.setOperation("取消报名");
            approvalInfo.setCreateBy(subject);
            approvalInfo.setCreateTime(new Date());

            approvalInfoService.insert(approvalInfo);

            messageResult.setData(applicationId);
            messageResult.setResult(true);
            messageResult.setMessage("取消报名成功");
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    private void saveAttachmentList(String applicationId,String studentId, List<AttachmentDTO> attachmentList, String subject){
        for (AttachmentDTO attachmentDTO : attachmentList) {
            studentFilesService.deleteByApplicationIdAndFileType(applicationId, attachmentDTO.getId());

            for (int i=0;i<attachmentDTO.getFiles().size();i++) {
                String fileUrl = attachmentDTO.getFiles().get(i);

                StudentFiles studentFiles = new StudentFiles();

                studentFiles.setId(UUID.randomUUID().toString());
                studentFiles.setApplicationId(applicationId);
                studentFiles.setStudentId(studentId);
                studentFiles.setFileType(attachmentDTO.getId());
                studentFiles.setFileUrl(fileUrl);
                studentFiles.setDelFlag(false);
                studentFiles.setSortNo(i+1);

                studentFiles.setCreateTime(new Date());
                studentFiles.setCreateBy(subject);

                studentFilesService.insert(studentFiles);
            }
        }
    }

    @PostMapping("detail")
    @ApiOperation(value = "报名信息")
    public MessageResult<Map> detail(String applicationId,@RequestAttribute String subject){
        MessageResult<Map> messageResult = new MessageResult<>();
        Map<String,Object> dataMap = new HashMap<>();

        try {
            //todo 从各张表中读取报名信息
            ApplicationPrimary applicationPrimary = applicationPrimaryService.get(applicationId);
            ApplicationMiddle applicationMiddle = applicationMiddleService.get(applicationId);
            String studentId = "";
            String schoolId = "";
            if (applicationPrimary != null){
                studentId = applicationPrimary.getStudentId();
                schoolId = applicationPrimary.getSchoolId();
            }else {
                studentId = applicationMiddle.getStudentId();
                schoolId = applicationMiddle.getSchoolId();
            }

            StudentInfo studentInfo = studentInfoService.get(studentId);
            SchoolInfo schoolInfo = schoolInfoService.get(schoolId);
            PersonStudent personStudent = personStudentService.findByPersonIdAndStudentId(subject,studentInfo.getId());

            ApplicationPrimaryDTO applicationPrimaryDTO = new ApplicationPrimaryDTO();

            if(applicationPrimary!=null) {
                applicationPrimaryDTO.setApplicationId(applicationId);
                applicationPrimaryDTO.setType(applicationPrimary.getType());
                applicationPrimaryDTO.setCategoryId(applicationPrimary.getCategoryId());
                applicationPrimaryDTO.setStreetId(applicationPrimary.getStreetId());
                applicationPrimaryDTO.setCommunityId(applicationPrimary.getCommunityId());
                applicationPrimaryDTO.setSchoolId(applicationPrimary.getSchoolId());
                applicationPrimaryDTO.setStudentId(applicationPrimary.getStudentId());
                applicationPrimaryDTO.setStatus(applicationPrimary.getStatus());
                applicationPrimaryDTO.setOfferCode(applicationPrimary.getOfferCode());
                applicationPrimaryDTO.setOfferImg(applicationPrimary.getOfferImg());
            }else {
                applicationPrimaryDTO.setApplicationId(applicationId);
                applicationPrimaryDTO.setType(applicationMiddle.getType());
                applicationPrimaryDTO.setCategoryId(applicationMiddle.getCategoryId());
                applicationPrimaryDTO.setStreetId(applicationMiddle.getStreetId());
                applicationPrimaryDTO.setCommunityId(applicationMiddle.getCommunityId());
                applicationPrimaryDTO.setSchoolId(applicationMiddle.getSchoolId());
                applicationPrimaryDTO.setStudentId(applicationMiddle.getStudentId());
                applicationPrimaryDTO.setChooseReason(applicationMiddle.getChooseReason());
                applicationPrimaryDTO.setStatus(applicationMiddle.getStatus());
                applicationPrimaryDTO.setOfferCode(applicationMiddle.getOfferCode());
                applicationPrimaryDTO.setOfferImg(applicationMiddle.getOfferImg());
            }

            if(studentInfo!=null){
                applicationPrimaryDTO.setName(studentInfo.getName());
                applicationPrimaryDTO.setOldName(studentInfo.getOldName());
                applicationPrimaryDTO.setCardType(studentInfo.getCardType());
                applicationPrimaryDTO.setIdCard(studentInfo.getIdCard());
                applicationPrimaryDTO.setDocumentUrl(studentInfo.getDocumentUrl());
                applicationPrimaryDTO.setSex(studentInfo.getSex());
                applicationPrimaryDTO.setBirthday(studentInfo.getBirthday());
                applicationPrimaryDTO.setNation(studentInfo.getNation());
                applicationPrimaryDTO.setAddress(studentInfo.getAddress());
                applicationPrimaryDTO.setResidenceAddress(studentInfo.getResidenceAddress());
                applicationPrimaryDTO.setStudentType(studentInfo.getType());
            }

            if (personStudent!=null) {
                applicationPrimaryDTO.setRelationship(personStudent.getRelationship());
            }

            if(schoolInfo!=null) {
                applicationPrimaryDTO.setSchoolName(schoolInfo.getName());
            }

            //房产证明异步读取

            //学生相关证明异步读取

            dataMap.put("application" ,applicationPrimaryDTO);

            //读取审批流程
            List<ApprovalInfo> approvalInfoList = approvalInfoService.findByApplicationId(applicationId);

            dataMap.put("approvalInfoList" ,approvalInfoList);

            messageResult.setData(dataMap);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }
}
