package com.jpsoft.campus.modules.mobile.controller;

import com.jpsoft.campus.modules.base.dto.SchoolInfoDTO;
import com.jpsoft.campus.modules.base.entity.EnrollControl;
import com.jpsoft.campus.modules.base.entity.SchoolInfo;
import com.jpsoft.campus.modules.base.entity.SchoolNumber;
import com.jpsoft.campus.modules.base.service.SchoolInfoService;
import com.jpsoft.campus.modules.base.service.SchoolNumberService;
import com.jpsoft.campus.modules.common.dto.MessageResult;
import com.jpsoft.campus.modules.sys.entity.DataDictionary;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;

/**
 * @author 墨鱼_mo
 * @date 2020-6-23 9:41
 */

@Slf4j
@RestController
@RequestMapping("/mobile/schoolInfoApi")
public class SchoolInfoApiController {

    @Autowired
    private SchoolInfoService schoolInfoService;
    @Autowired
    private SchoolNumberService schoolNumberService;




    @PostMapping("typeList")
    @ApiOperation(value = "手机端学校列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "type", value = "类型（0（全部）/1（小学）/2（中学））", required = true, paramType = "form")
    })
    public MessageResult<Map> typeList(String type){
        MessageResult<Map> messageResult = new MessageResult<>();

        try {
            List<SchoolInfo> schoolInfoList = null;

            if (!"0".equals(type)){
                schoolInfoList =  schoolInfoService.findByEnrollmentType(type);
            }else {
                schoolInfoList = schoolInfoService.list();
            }
            List<SchoolInfoDTO> list = new ArrayList<>();
            if (schoolInfoList.size()>0){
                for (SchoolInfo schoolInfo : schoolInfoList){
                    SchoolInfoDTO schoolInfoDTO = new SchoolInfoDTO();
                    BeanUtils.copyProperties(schoolInfo,schoolInfoDTO);
                    list.add(schoolInfoDTO);
                }
            }
            HashMap map = new HashMap();
            map.put("list",list);
            messageResult.setCode(200);
            messageResult.setData(map);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setCode(500);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @PostMapping("detail")
    @ApiOperation(value = "手机端学校列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "id", required = true, paramType = "form")
    })
    public MessageResult<SchoolInfoDTO> detail(String id){
        MessageResult<SchoolInfoDTO> messageResult = new MessageResult<>();

        try {

            SchoolInfo schoolInfo = schoolInfoService.get(id);
            SchoolInfoDTO schoolInfoDTO = new SchoolInfoDTO();
            if (schoolInfo !=null){
                BeanUtils.copyProperties(schoolInfo,schoolInfoDTO);
            }

            messageResult.setCode(200);
            messageResult.setData(schoolInfoDTO);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setCode(500);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }


    @GetMapping("queryTransferSchoolList")
    @ApiOperation(value = "查询转校学校列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "enrollmentType", value = "报名类型(1小学,2初中)", required = true, paramType = "query")
    })
    public MessageResult<List> queryTransferSchoolList(String enrollmentType,String token){
        MessageResult<List> messageResult = new MessageResult<>();

        try {
            List<SchoolInfo> schoolInfoList = schoolInfoService.findByEnrollmentType(enrollmentType);
            List<SchoolInfo> returnList = new ArrayList<>();

            for (SchoolInfo schoolInfo : schoolInfoList) {
                List<SchoolNumber> schoolNumberList = schoolNumberService.findHaveNumberBySchoolId(enrollmentType, schoolInfo.getId());
                if (schoolNumberList.size() > 0) {
                    schoolInfo.setSchoolNumbers(schoolNumberList);
                    returnList.add(schoolInfo);
                }

                messageResult.setData(schoolInfoList);
                messageResult.setResult(true);
            }
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @GetMapping("querySchoolGradeNumber")
    @ApiOperation(value = "查询学校的年级以及有多少空位")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "schoolId", value = "学校ID", required = true, paramType = "query"),
            @ApiImplicitParam(name = "enrollmentType", value = "报名类型(1小学,2初中)", required = true, paramType = "query")
    })
    public MessageResult<List> querySchoolGradeNumber(String schoolId,String enrollmentType,String token){
        MessageResult<List> messageResult = new MessageResult<>();

        try {
            List<SchoolNumber> schoolNumberList = schoolNumberService.findHaveNumberBySchoolId(enrollmentType,schoolId);

            messageResult.setData(schoolNumberList);
            messageResult.setResult(true);
        }
        catch (Exception ex){
            log.error(ex.getMessage(),ex);

            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }

    @GetMapping("transferSchoolControl")
    @ApiOperation(value = "插班转学打开关闭控制")
    public MessageResult<Boolean> transferSchoolControl(String token){
        MessageResult<Boolean> messageResult = new MessageResult<>();

//        try {
//               int schoolNumber = schoolNumberService.findNumByHaveNumber();
//                if (schoolNumber > 0) {
//                    messageResult.setData(true);
//                }else{
//                    messageResult.setData(false);
//                }
//                messageResult.setResult(true);
//        }
//        catch (Exception ex){
//            log.error(ex.getMessage(),ex);
//            messageResult.setData(false);
//            messageResult.setResult(false);
//            messageResult.setMessage(ex.getMessage());
//        }
//
//        return messageResult;

        boolean result = true;
        try {
            List<SchoolNumber> numberList = schoolNumberService.findAllHaveNumber();
            int i = 0;
            for (SchoolNumber sn : numberList){
                if (sn.getStartTime().compareTo(new Date()) < 0 && sn.getEndTime().compareTo(new Date()) > 0) {
                    i++;
                }
            }
            if (i < 1){
                result = false;
            }

            messageResult.setCode(200);
            messageResult.setData(result);
            messageResult.setResult(true);
        } catch (Exception ex) {
            log.error(ex.getMessage(), ex);

            messageResult.setData(false);
            messageResult.setCode(500);
            messageResult.setResult(false);
            messageResult.setMessage(ex.getMessage());
        }

        return messageResult;
    }
}
