package com.jpsoft.campus.modules.wechat.controller;

import com.jpsoft.campus.config.WechatConfig;
import com.jpsoft.campus.modules.common.dto.MessageResult;
import com.jpsoft.campus.modules.common.utils.Sign;
import com.jpsoft.campus.modules.common.utils.WeixinUtil;
import com.jpsoft.campus.modules.wechat.entity.AccessToken;
import com.jpsoft.campus.modules.wechat.vo.UserInfo;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.SetOperations;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Slf4j
@RequestMapping("/wechat")
@RestController
public class WxController {
    @Autowired
    private WechatConfig wechatConfig;

    @Autowired
    private RedisTemplate<String,Object> redisTemplate;

    @Autowired
    private SetOperations<String,Object> setOperations;

    @ApiOperation(value = "获取微信配置")
    @GetMapping(value = "/getConfig")
    public MessageResult getConfig(String url) {
        int code = 200;
        String message = "获取成功";
        Object data = "";
        boolean result = true;

        try {
            AccessToken token = WeixinUtil.getCommonAccessToken(wechatConfig.getTokenUrl(),wechatConfig.getAppId(), wechatConfig.getAppSecret());

            Map<String, String> wxMap = Sign.sign(WeixinUtil.getJsAPI(token.getToken()), url);

            wxMap.put("appId", wechatConfig.getAppId());

            HashMap<String, Object> dataMap = new HashMap<String, Object>();

            dataMap.put("wxConfig", wxMap);

            data = dataMap;
        } catch (Exception ex) {
            code = 500;
            result = false;
            message = "系统错误";
        }

        return new MessageResult(result, message, data, code);
    }

    @ApiOperation(value = "获取公众号用户信息")
    @GetMapping(value = "findUserInfo/{code}")
    public MessageResult findUserInfo(@PathVariable String code) {
        try {
            String key = "wx_userInfo";

            if (!redisTemplate.hasKey(key)){
                setOperations.add(key, code);
                redisTemplate.expire(key,6, TimeUnit.HOURS);
            }
            else {
                if (!setOperations.isMember(key, code)) {
                    setOperations.add(key, code);
                } else {
                    log.error(code + "已使用!");
                    return new MessageResult(false, "获取微信信息失败", "", 500);
                }
            }

            AccessToken at = WeixinUtil.getAccessToken(wechatConfig.getAppId(), wechatConfig.getAppSecret(), code);

            if (at != null && StringUtils.isNotBlank(at.getOpenid())) {
                String openId = at.getOpenid();
                System.out.println("openId:" + openId);

                UserInfo userInfo = WeixinUtil.getUserInfo(openId, at.getToken());

                HashMap<String, Object> dataMap = new HashMap<String, Object>();

                dataMap.put("userInfo", userInfo);

                return new MessageResult(true, "获取微信信息成功", userInfo, 200);

            } else {
                return new MessageResult(false, "获取微信信息失败", "", 400);
            }

        } catch (Exception ex) {
            log.error(ex.getMessage(),ex);

            return new MessageResult(false, "系统错误", "", 500);
        }
    }
}
